/* Copyright (c) 2001, Nokia Mobile Phones. All rights reserved */

/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#ifndef __SOCKETSWRITE_H__
#define __SOCKETSWRITE_H__

#include <in_sock.h>
#include "TimeOutNotify.h"
#include "uinotify.h"

class CTimeOutTimer;


/*! 
  @class CSocketsWrite
  
  @discussion This class handles writing data to the socket.
  Data to be written is accumulated in iTransferBuffer, and is then transferred
  to iWriteBuffer for the actual write to the socket.
  */
class CHttpWrite : public CActive, public MTimeOutNotify
{
public:
/*!
  @function NewL
   
  @discussion Create a CSocketsWrite object
  @param aConsole console to use for ui output
  @param aSocket socket to write to
  @result a pointer to the created instance of CSocketsWrite
  */
    static CHttpWrite* NewL(MHttpNotify& aConsole, RSocket& aSocket);

/*!
  @function NewLC
   
  @discussion Create a CSocketsWrite object
  @param aConsole console to use for ui output
  @param aSocket socket to write to
  @result a pointer to the created instance of CSocketsWrite
  */
    static CHttpWrite* NewLC(MHttpNotify& aConsole, RSocket& aSocket);

/*!
  @function ~CSocketsWrite
  
  @discussion Destroy the object and release all memory objects
  */
    ~CHttpWrite();

/*!
  @function IssueWrite

  @discussion Write the data to the socket (buffered)
  @param aData the data to be written
  */
    void IssueWriteL(const TDesC8& aData);

protected: // from CActive
/*!
  @function DoCancel
   
  @discussion cancel any outstanding operation
  */
    void DoCancel();

/*!
  @function RunL
   
  @discussion called when operation complete
  */
    void RunL();    

    // From MTimeOutNotify
    void TimerExpired(); 

private:
/*!
  @function CSocketsWrite
  
  @discussion Perform the first phase of two phase construction 
  @param aConsole console to use for ui output
  @param aSocket socket to read from
  */
    CHttpWrite(MHttpNotify& aConsole, RSocket& aSocket);

/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CSocketsWrite 
  */
    void ConstructL();

/*!
  @function SendNextPacket

  @discussion Handle a 'write buffer empty' situation.
  */    
    void SendNextPacket();


private: // Member data
    enum TWriteState 
    {
        ESending, EWaiting, ECommsFailed
    };

    enum
    {
        KWriteBufferSize = 128   
    };

    // Member variables
    RSocket&                iSocket;
    MHttpNotify&              iNotify;
    TBuf8<KWriteBufferSize> iTransferBuffer; // Accumulate data to send in here
    TBuf8<KWriteBufferSize> iWriteBuffer; // Holds data currently being sent to socket
    CTimeOutTimer*          iTimer;
    TInt                    iTimeOut;
    TWriteState             iWriteStatus;
};

#endif // __SOCKETSWRITE_H__

