
/*
 * Copyright Justin Armstrong 2008. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#ifndef LOCATIONDB_H
#define LOCATIONDB_H

#include <aknapp.h>
#include <aknappui.h>
#include <e32def.h>
#include <D32DBMS.H> 
#include <lbsposition.h> 

#include "stringintdict.h"
#include "weatherinfo.h"

struct TLocationDescription
    {
    TBuf<80> iPlaceName;
    TBuf<80> iState;
    TBuf<80> iCountry;
    TBuf<4> iIcao;
    TCoordinate iCoordinate;
    
    TReal64 iDistance;    //only used when returning list of locations near to a specific point
    static TInt CompareByDistance( const TLocationDescription& loc1, const TLocationDescription& loc2 )
        {
        return loc1.iDistance > loc2.iDistance;
        }
    };


class CLocationDB : public CBase  
    {
    public:

        CLocationDB();
        virtual ~CLocationDB();

        void ConstructL(CCoeEnv* aCoeEnv);
        
        void CreateLocationsDbFromLocationsFileL(TFileName& aDbFileName, RFile& aLocationsFile);
        void OpenLocationsDbL(TFileName& aDbFileName);
    
        CDesCArrayFlat* GetAllCountriesL();
        CDesCArrayFlat* GetStatesInCountryL(TPtrC aCountry);
        RPointerArray<TLocationDescription>* GetLocationsInCountryL(TPtrC aCountry, TPtrC aState);
        RPointerArray<TLocationDescription>* FindNearbyLocationsL(TReal64 aLatitude, TReal64 aLongitude);
        bool GetLocationDescriptionByIcao(TPtrC aIcao, TLocationDescription* aLocDesc);
        TInt Close();
        
    private:
        
        void ParseLocationsFileL(RFile& aLocationsFile);
        void CreateLocationsDbL(TFileName& aDbFileName);
        
        void CreateTablesL();
        void CreateIndicesL();
        TReal64 ParseLatitude(TPtrC string);
        TReal64 ParseLongitude(TPtrC string);        
        TPtrC GetNextElement(TDesC& aLine, TInt& aStart);

    
    private:
    
        RFs              iFsSession;
        RDbStoreDatabase iDb;
        CFileStore*      iFileStore;
        TBool            iOpen;
        
    };

#endif
