

#ifndef LOCATION_VIEW_CONTROLLER_H
#define LOCATION_VIEW_CONTROLLER_H

// INCLUDES
#include <aknview.h>
#include "location.h"
#include "locationcontainer.h"

// FORWARD DECLARATIONS
//class CLocationContainer;


// CLASS DECLARATION

class CLocationViewController: public CAknView
    {
    public:   // Constructors and destructor

        /**
        * NewL.
        * Two-phased constructor.
        * Construct a CLocationViewController for the AVKON application aApp.
        * Using two phase construction,and return a pointer to the created object
        * @return a pointer to the created instance of CLocationViewController
        */
        static CLocationViewController* NewL();

        /**
        * NewLC.
        * Two-phased constructor.
        * Construct a CLocationViewController for the AVKON application aApp.
        * Using two phase construction,and return a pointer to the created object
        * @return a pointer to the created instance of CLocationViewController
        */
        static CLocationViewController* NewLC();

        /**
        * ~CLocationViewController.
        * Virtual Destructor.
        */
        virtual ~CLocationViewController();

    public: // Functions from base classes from CAknView

        /**
        * Id
        * @return Id Uid value
        */
        TUid Id() const;

        /**
        * HandleCommandL
        * From CAknView, takes care of command handling.
        * @param aCommand Command to be handled
        */
        void HandleCommandL( TInt aCommand );
        
        /**
        * HandleSizeChange
        * Called by HandleResourceChangeL() from CMultiViewsAppUi when layout 
        * is changed.
        * @param aType Type of resources that have changed
        */
        void HandleSizeChange( TInt aType );

        /**
        * DoActivateL
        * From CAknExView, activate an AknView.
        * @param aPrevViewId The id of the previous view
        * @param aCustomMessageId message identifier
        * @param aCustomMessage custom message provided when the view is changed
        */
        void DoActivateL( const TVwsViewId& aPrevViewId,
                          TUid aCustomMessageId,
                          const TDesC8& aCustomMessage );

        /**
        * DoDeactivate
        * From AknView, deactivate an AknView
        * Remove the container class instance from the App UI's stack and
        * deletes the instance
        */
        void DoDeactivate();
        
        void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);

    public: //custom functions
        void SetLocation(CLocation* aLocation);
        
    private:   // Constructors and destructor

        /**
        * CLocationViewController.
        * C++ default constructor.
        */
        CLocationViewController();

        /**
        * ConstructL.
        * 2nd phase constructor.
        */
        void ConstructL();

    private: // Data

        CLocationContainer* iContainer; 
        
        CLocation* iLocation;

        /** View Identifier **/
        TUid iIdentifier;
    };


#endif // LOCATION_VIEW_CONTROLLER_H


// End of File
