/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef WEATHERCONTAINER_H
#define WEATHERCONTAINER_H

// INCLUDES
#include <coecntrl.h>
#include <coeccntx.h>
#include <aknappui.h>
#include <eiksbfrm.h>
#include <eiksbobs.h>
#include <eikapp.h>
#include "stringintdict.h"
#include "bitmaplabel.h"
#include "weatherinfo.h"
#include "location.h"
#include "weatherutils.h"
#include "weatherdisplay.h"

class CEikLabel;
class CWeatherDisplay;
class CLocation;

class CWeatherContainer : public CCoeControl, MCoeControlObserver, MEikScrollBarObserver
    {
    public:
        
        CWeatherContainer();
        
        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CWeatherContainer();

    public: // New functions


        void SetLocation(CLocation* aLocation);
        void UpdateScrollbars();


    public: // Functions from base classes

        void HandleResourceChange(TInt aType);
        TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;

       /**
        * From ?base_class ?member_description
        */
        // event handling section
        // e.g Listbox events
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
        
        void HandleScrollEventL(CEikScrollBar* aScrollBar, TEikScrollEvent aEventType);

        TTypeUid::Ptr MopSupplyObject(TTypeUid aId);        

    private:


    private: //data
        

        CAknsBasicBackgroundControlContext* iBgContext;

        CEikScrollBarFrame* iSBFrame;

        TInt iYTop;

        CWeatherDisplay* iWeatherDisplay;

    };

#endif

// End of File
