
/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#ifndef WEATHER_DISPLAY_H
#define WEATHER_DISPLAY_H

// INCLUDES
#include <coecntrl.h>
#include <coeccntx.h>
#include <aknappui.h>
#include <eiksbfrm.h>
#include <eiksbobs.h>
#include <eikapp.h>
#include "stringintdict.h"
#include "bitmaplabel.h"
#include "weatherinfo.h"
#include "weatherutils.h"

class CEikLabel;
class CLocation;

class CWeatherDisplay : public CCoeControl
    {
    public:
        
        CWeatherDisplay();
        
        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CWeatherDisplay();

    public: // New functions

        void SetLocation(CLocation* aLocation);
        void ReLayout();

    public: // Functions from base classes


    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;

        
    private:

    	void AddLabelL(const TDesC& aString, TPoint& aPos, TInt aWidth);
    	void AddConditionLabelL(TWeatherCondition&, TPoint& aPos, TInt aWidth);
    	void AddCloudLabelL(TCloud& cloud, TPoint& aPos, TInt aWidth);
    	void AddCloudSummaryLabelL(CArrayFixFlat<TCloud>* aClouds, TPoint& aPos, TInt aWidth);

    	void AddWindAndVisibilitySection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth);
    	void AddConditionSection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth);
    	void AddCloudSection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth);
    	void AddHumidityDewPointAndPressureSection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth);
    	
    private: //data
        

    	CLocation* iLocation;
    	
        TRect iMaxRect;
        
        const CFont* iDenseFont;
        CArrayPtrFlat<CCoeControl>* iControlArray;

        bool iDrawSplitters;

        TInt iXPosMidSplit;	//top and mid boxes are split in two if in landscape mode
        TInt iTopBoxYBottom; //temp on LHS, 2 boxes below on RHS
        TInt iWindBoxYBottom; //wind info
        TInt iVisibilityBoxYBottom;  //visibility info
        
        TInt iMidBoxYBottom; //the midbox contains conditions on LHS, and the 3 smaller boxes below on RHS
        TInt iHumidityBoxYBottom; //box to store wind info
        TInt iDewpointBoxYBottom; //dewpoint info
        TInt iPressureBoxYBottom; //pressure info
        //clouds appear below this
        
        
        TBuf<100> iTempDesc1;
        TBuf<100> iTempDesc2;
        
        CStringIntDict* iPhenomenonDict;
        CStringIntDict* iModifierDict;
        CStringIntDict* iIntensityDict;

        CStringIntDict* iCloudDict;

        CStringIntDict* iCompassDict;

    };

#endif

// End of File
