
/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#ifndef WEATHER_INFO_H
#define WEATHER_INFO_H

#include <aknapp.h>
#include "constants.h"

struct TWeatherCondition
    {
    TIntensity iIntensity;
    TPhenomenonModifier iModifier;
    TPhenomenon iPhenomenon;
    TPhenomenonTrend iTrend;
    };

struct TCloud
    {
    TCloudCoverage iCoverage;
    TInt iHeight;
    };



class CWeatherInfo : public CBase  
{
public:
	CWeatherInfo();
	virtual ~CWeatherInfo();

    static CWeatherInfo*  NewLC();

    /////////////////setters

    void AddConditionL(TPhenomenonTrend aTrend, 
					   TIntensity aIntensity, 
                       TPhenomenonModifier aModifier,
                       TPhenomenon iPhenomenon);


    void AddCloudL(TCloudCoverage aCoverage, TInt aHeight);

    void SetWind(TInt aWindSpeed, TCompassDirection aWindDirection, 
                           TInt aGustSpeed, TBool aDirectionVariable);


    void SetVisibility(TInt aVisibility);

    void SetTemperature(TInt aTemp);
    void SetDewPoint(TInt aDewPoint);

    void SetTime(TTime& aTime);

    void SetPressure(TInt aPressure);


    /////////////////getters

    CArrayFixFlat<TWeatherCondition>* GetConditions();
    CArrayFixFlat<TCloud>* GetClouds();

    //these functions return false if the attribute is not present in the report
    //otherwise return true and set the by-ref args
    TBool GetWind(TInt& aWindSpeed, TCompassDirection& aWindDirection, 
                  TInt& aGustSpeed, TBool& aDirectionVariable);

    TBool GetVisibility(TInt& aVisibility);
    TBool GetTemperature(TInt& aTemperature);
    TBool GetDewPoint(TInt& aDewPoint);
    TBool GetPressure(TInt& aPressure);
    TBool GetTime(TTime& aTime);


private:

    TInt iWindSpeed;    //knots
    TCompassDirection iWindDirection;
    TInt iGustSpeed;    //knots

    TBool iHasWind;     //false if report contains no wind details
    TBool iDirectionVariable;

    TInt iTemperature;
    TBool iHasTemperature;

    TInt iDewPoint;
    TBool iHasDewPoint;
    
    TInt iPressure; //millibars
    TBool iHasPressure;

    TInt iVisibility;   //metres
    TBool iHasVisibility;

    CArrayFixFlat<TWeatherCondition>* iConditions;

    CArrayFixFlat<TCloud>* iClouds;

    TTime iReportTime;


};

#endif
