

#ifndef WEATHER_VIEW_CONTROLLER_H
#define WEATHER_VIEW_CONTROLLER_H

// INCLUDES
#include <aknview.h>
#include "location.h"
#include "weathercontainer.h"
// FORWARD DECLARATIONS
class CWeatherContainer;
class CLocation;


// CLASS DECLARATION

class CWeatherViewController: public CAknView
    {
    public:   // Constructors and destructor

        /**
        * NewL.
        * Two-phased constructor.
        * Construct a CWeatherViewController for the AVKON application aApp.
        * Using two phase construction,and return a pointer to the created object
        * @return a pointer to the created instance of CWeatherViewController
        */
        static CWeatherViewController* NewL();

        /**
        * NewLC.
        * Two-phased constructor.
        * Construct a CWeatherViewController for the AVKON application aApp.
        * Using two phase construction,and return a pointer to the created object
        * @return a pointer to the created instance of CWeatherViewController
        */
        static CWeatherViewController* NewLC();

        /**
        * ~CWeatherViewController.
        * Virtual Destructor.
        */
        virtual ~CWeatherViewController();

    public: // Functions from base classes from CAknView

        /**
        * Id
        * @return Id Uid value
        */
        TUid Id() const;

        /**
        * HandleCommandL
        * From CAknView, takes care of command handling.
        * @param aCommand Command to be handled
        */
        void HandleCommandL( TInt aCommand );
        
        /**
        * HandleSizeChange
        * Called by HandleResourceChangeL() from CMultiViewsAppUi when layout 
        * is changed.
        * @param aType Type of resources that have changed
        */
        void HandleSizeChange( TInt aType );

        /**
        * DoActivateL
        * From CAknExView, activate an AknView.
        * @param aPrevViewId The id of the previous view
        * @param aCustomMessageId message identifier
        * @param aCustomMessage custom message provided when the view is changed
        */
        void DoActivateL( const TVwsViewId& aPrevViewId,
                          TUid aCustomMessageId,
                          const TDesC8& aCustomMessage );

        /**
        * DoDeactivate
        * From AknView, deactivate an AknView
        * Remove the container class instance from the App UI's stack and
        * deletes the instance
        */
        void DoDeactivate();
        
        void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);

    public: //custom functions
    	void SetLocation(CLocation* aLocation);
        
    private:   // Constructors and destructor

        /**
        * CWeatherViewController.
        * C++ default constructor.
        */
        CWeatherViewController();

        /**
        * ConstructL.
        * 2nd phase constructor.
        */
        void ConstructL();

    private: // Data

    	CWeatherContainer* iContainer; 
    	
    	CLocation* iLocation;

        /** View Identifier **/
        TUid iIdentifier;
    };


#endif // WEATHER_VIEW_CONTROLLER_H


// End of File
