
/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <akniconutils.h>
#include <aknutils.h> 
#include "bitmaplabel.h"
#include "weatherutils.h"
#include <aknsutils.h> 

#include <aknglobalnote.h> 
#include <BAUTILS.H>

#define PIXELS_ABOVE_TEXT 10

CBitmapLabel::CBitmapLabel()
    {

    }

CBitmapLabel::~CBitmapLabel()
    {
    //delete iBgBitmap;
    //delete iBgMask;
    if (iBitmap) delete iBitmap;
    if (iMask) delete iMask;
    
    delete iWrappedTextArray;
    }


void CBitmapLabel::SetContainerWindowL(const CCoeControl& aContainer)
    {
    CCoeControl::SetContainerWindowL(aContainer);

    }


void CBitmapLabel::ConstructL(CFbsBitmap* aBitmap, CFbsBitmap* aMask)
    {
    iBitmap = aBitmap;
    iMask = aMask;
    
    SetSize(iBitmap->SizeInPixels());
    ActivateL();
    
    }

void CBitmapLabel::ConstructL(const CFont* aFont, const TDesC& aString, TUint aWidth, TInt aBitmapId)
    {
    
    iString = aString;
    iFont = aFont;
    
    TSize bitmapSize;
    
    TInt pixelPadding = 0;
    if (aBitmapId > 0)
        {
        TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();    
        TParsePtr parser(app);
        TFileName mbmFile = parser.Drive();
        mbmFile.Append(_L("\\resource\\apps\\weather.mbm")); 
        
        iBitmap = AknIconUtils::CreateIconL(mbmFile, aBitmapId);
        iMask = AknIconUtils::CreateIconL(mbmFile, aBitmapId+2);
        
        bitmapSize = iBitmap->SizeInPixels();
        pixelPadding = PIXELS_ABOVE_TEXT;
        }


    TInt textWidth = aWidth - (bitmapSize.iWidth + 10);
    iWrappedTextArray = new (ELeave)CArrayFixSeg<TPtrC>( 2 );
    CleanupStack::PushL(iWrappedTextArray);
    AknTextUtils::WrapToArrayL(iString, textWidth, *aFont, *iWrappedTextArray);
    TInt height = Max(bitmapSize.iHeight, aFont->FontMaxHeight()*iWrappedTextArray->Count() + pixelPadding);
    SetSize(TSize(aWidth, height));

    //ActivateL();
    
    CleanupStack::Pop();
    
    }

CCoeControl* CBitmapLabel::ComponentControl(TInt /*aIndex*/) const
    {

    return NULL;

    }


TInt CBitmapLabel::CountComponentControls() const
    {

    return 0;

    }

void CBitmapLabel::Draw(const TRect& aRect) const
    {
    CWindowGc& gc = SystemGc();

    TRgb colour;
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    AknsUtils::GetCachedColor(skin, colour, KAknsIIDQsnTextColors, EAknsCIQsnTextColorsCG6);
    
    gc.SetPenColor(colour);
    gc.SetPenStyle(CGraphicsContext::ENullPen);
    //gc.SetPenStyle(CGraphicsContext::ESolidPen);
    gc.SetBrushStyle( CGraphicsContext::ENullBrush);
        
    TRect bitmapRect;
    TSize bitmapSize;
    if (iBitmap)
        {
        bitmapSize = iBitmap->SizeInPixels();
        //bitmapRect = TRect(aRect.iTl, bitmapSize);
        gc.BitBltMasked(aRect.iTl, iBitmap, TRect(TPoint(0,0), bitmapSize), iMask, ETrue);
        }
    
    if (iString.Length() != 0)
        {
        gc.SetPenStyle(CGraphicsContext::ESolidPen);
        gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
        gc.UseFont(iFont);
        TPoint textPos = aRect.iTl;
        textPos.iX += iBitmap ? bitmapSize.iWidth + 10 : 0;
        textPos.iY += iFont->FontMaxAscent();
        TInt i;
        for (i=0; i < iWrappedTextArray->Count(); i++) 
            {
            TPtrC line = (*iWrappedTextArray)[i];
            gc.DrawText(line, textPos);
            textPos.iY += iFont->HeightInPixels();
            }
    
        }

    }

TSize CBitmapLabel::MinimumSize()
    {
    
    TSize size = Size();    //just return the size we set in ConstructL
    return size;
    }


void CBitmapLabel::SizeChanged()
    {
    //DrawDeferred();

    }
