/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

/* Based on Nokia sample code */
/* Copyright (c) 2001, Nokia Mobile Phones. All rights reserved */


#include "httpwrite.h"
#include "TimeOutTimer.h"
#include "Sockets.pan"
#include "UINotify.h"

static const TInt KTimeOut = 30000000; // 30 seconds time-out

CHttpWrite* CHttpWrite::NewL(MHttpNotify& aConsole, RSocket& aSocket)
    {
    CHttpWrite* self = CHttpWrite::NewLC(aConsole, aSocket);
    CleanupStack::Pop();
    return self;
    }

    
CHttpWrite* CHttpWrite::NewLC(MHttpNotify& aConsole, RSocket& aSocket)
    {
    CHttpWrite* self = new (ELeave) CHttpWrite(aConsole, aSocket);
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
    }


CHttpWrite::CHttpWrite(MHttpNotify& aNotify, RSocket& aSocket)
: CActive(EPriorityStandard),
  iSocket(aSocket),
  iNotify(aNotify)
    {
    }


CHttpWrite::~CHttpWrite()
    {
    Cancel();
    delete iTimer;
    iTimer = NULL;
    }

void CHttpWrite::DoCancel()
    {    
    // Cancel asychronous write request
    iSocket.CancelWrite();
    }

void CHttpWrite::ConstructL()
    {
    CActiveScheduler::Add(this);

    iTimeOut = KTimeOut; 
    iTimer = CTimeOutTimer::NewL(10, *this);
    iWriteStatus = EWaiting;
    }

void CHttpWrite::TimerExpired()
    {
    Cancel();
    iWriteStatus = ECommsFailed;
    iNotify.SocketStatusNotify(_L("Write operation timed out"), EFalse);
    }

void CHttpWrite::RunL()
    {
    // Active object request complete handler
    if (iStatus == KErrNone)
        {
        switch(iWriteStatus)
            {
            // Character has been written to socket
            case ESending:
                SendNextPacket();
                break;
            default:
                User::Panic(KPanicSocketsEngineWrite, ESocketsBadStatus);
                break;
            };
        }
    else 
        {
        // Error: pass it up to user interface
        iTimer->Cancel();
        iNotify.SocketStatusNotify(_L("CSocketsWrite error"), EFalse);
        iWriteStatus = ECommsFailed;
        }
    }

void CHttpWrite::IssueWriteL(const TDesC8& aData)
    {
    if ((iWriteStatus != EWaiting) && (iWriteStatus != ESending))
        {
        User::Leave(KErrNotReady);
        }

    // Write data to a stream socket
    if ((aData.Length() + iTransferBuffer.Length()) > iTransferBuffer.MaxLength())
        {
        // Not enough space in buffer
        User::Leave(KErrOverflow);
        }

    // Add new data to buffer
    iTransferBuffer.Append(aData);

    if (!IsActive())
        {
        SendNextPacket();
        }
    }

void CHttpWrite::SendNextPacket()
    {
    iTimer->Cancel(); // Cancel TimeOut timer
    iWriteStatus = EWaiting;

    if (iTransferBuffer.Length() > 0)
        {
        // Move data from transfer buffer to actual write buffer
        iWriteBuffer = iTransferBuffer;
        iTransferBuffer.Zero();
        iSocket.Write(iWriteBuffer, iStatus); // Initiate actual write
    
        // Request timeout
        iTimer->After(iTimeOut);
        SetActive();
        iWriteStatus = ESending;
        }
    else
        {
        //iNotify.SocketStatusNotify(_L("Finished writing"), EFalse);
        }
    }
