
/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include "location.h"


CLocation::CLocation(TLocationDescription& aDescription)
    {
    iDescription = aDescription;
    }


CLocation::CLocation() : iDummyReport(_L(""))
    {
    }

void CLocation::ConstructFromStreamL(CLocationDB* aLocationDB, RDictionaryReadStream& aReadStream)
    {

    //aReadStream >> iDescription.iPlaceName;    //TODO we don't need to store this
    aReadStream >> iDescription.iIcao;

    aLocationDB->GetLocationDescriptionByIcao(iDescription.iIcao, &iDescription);
    
    TBool hasReport = aReadStream.ReadInt16L();
    if (hasReport)
        {
        iLastReportString = HBufC::NewL(aReadStream, 1000);
        iLastReport = GLOBALS->iParser->ParseReportL(*iLastReportString);
        }

    }

void CLocation::ParseReportL(HBufC* reportString)
    {
    
    if (iLastReportString) delete iLastReportString;
    if (iLastReport) delete iLastReport;
    iLastReportString = reportString;
    iLastReport = GLOBALS->iParser->ParseReportL(*reportString);

    }

void CLocation::SaveL(RDictionaryWriteStream& writeStream)
    {

    //writeStream << iDescription.iPlaceName;
    writeStream << iDescription.iIcao;

    if (iLastReportString)
        {
        writeStream.WriteInt16L(ETrue);
        writeStream << *iLastReportString;

        }
    else
        {
        writeStream.WriteInt16L(EFalse);
        }

    }

TDesC& CLocation::Name()
    {
    return iDescription.iPlaceName;
    }

TDesC& CLocation::IcaoCode()
    {
    return iDescription.iIcao;
    }

CWeatherInfo* CLocation::Report()
    {
    return iLastReport;
    }

TDesC& CLocation::GetReportString()
    {
    if (iLastReportString)
        {
        return *iLastReportString;
        }
    else 
        {
        return iDummyReport;
        }
    }

CLocation::~CLocation()
    {
    delete iLastReportString;
    delete iLastReport;
    }
