/*
 * Copyright Justin Armstrong 2008. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#include <aknscontrolcontext.h> 
#include <aknsbasicbackgroundcontrolcontext.h> 
#include <aknnotewrappers.h> 
#include <aknglobalnote.h> 
#include <e32math.h>
#include "locationcontainer.h"

CLocationContainer::CLocationContainer()
    {

    }

CLocationContainer::~CLocationContainer()
    {
    
    delete iBgContext;
    
    delete iListBox;
    
    delete iIcaoHeading;
    delete iCountryHeading;
    delete iLatitudeHeading;
    delete iLongitudeHeading;
    }


void CLocationContainer::ConstructL(const TRect& aRect)
    {

    CreateWindowL();
    
    iBgContext = CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain,aRect,ETrue);
    
    TResourceReader rr;
    iEikonEnv->CreateResourceReaderLC(rr, R_LOCATION_INFO_LISTBOX);

    
    //iListBox = new (ELeave) CAknFormDoubleStyleListBox();
    iListBox = new (ELeave) CAknSingleHeadingStyleListBox();
    iListBox->SetContainerWindowL(*this);
    iListBox->ConstructFromResourceL(rr);
    CleanupStack::PopAndDestroy();
    


    iIcaoHeading = iCoeEnv->AllocReadResourceL(R_TEXT_ICAO);
    iCountryHeading = iCoeEnv->AllocReadResourceL(R_TEXT_COUNTRY);
    iLatitudeHeading = iCoeEnv->AllocReadResourceL(R_TEXT_LATITUDE);
    iLongitudeHeading = iCoeEnv->AllocReadResourceL(R_TEXT_LONGITUDE);
    
    MDesCArray* itemList = iListBox->Model()->ItemTextArray();
    iItemArray = (CDesCArray*) itemList;   
    
    SetRect(aRect);
    ActivateL();
    }


void CLocationContainer::AddRowL(const TDesC& aLabel, const TDesC& aValue)
    {
    TBuf<100> rowBuffer;
    _LIT(KTab, "\t");
    
    rowBuffer.Zero();
    //rowBuffer.Append(KTab);  //enable for CAknFormDoubleStyleListBox
    rowBuffer.Append(aLabel);
    rowBuffer.Append(KTab);
    rowBuffer.Append(aValue);
    rowBuffer.Append(KTab);
    rowBuffer.Append(KTab);
    
    iItemArray->AppendL(rowBuffer);
    }



// Degrees sign delimeter used in formatting methods
_LIT(KDelimDegree,"\xb0"); // "" symbol

// Dot delimeter used in formatting methods
_LIT(KDelimDot,"\x2e"); // "." symbol

// Plus sign delimeter used in formatting methods
_LIT(KDelimPlus,"\x2b"); // "+" symbol

// Minus sign delimeter used in formatting methods
_LIT(KDelimMinus,"\x2d"); // "-" symbol

// Quotation sign delimeter used in formatting methods
_LIT(KDelimQuot,"\x22"); // "\"" symbol

// Apostrophe sign delimeter used in formatting methods
_LIT(KApostrophe,"\x27"); // "'" symbol

// Not-a-number string
_LIT(KNan,"NaN");
// -----------------------------------------------------------------------------
// CLbsReferenceContainer::GetDegreesString
// Changes latitude or longitude represented as degrees to a string of 
// a form +DDD'MM''SS.SSSS
// -----------------------------------------------------------------------------
//

void CLocationContainer::GetDegreesString(const TReal64& aDegrees, TDes& aDegreesString) const
    {
    const TReal KSecondsInMinute = 60.0;
    const TInt KNumWidth = 3;
    
    // If the aDegree is a proper number
    if ( !Math::IsNaN(aDegrees) )
        {
        // Integer part of the degrees
        TInt intDegrees = static_cast<TInt>(aDegrees);

        // Positive float of the degrees
        TReal64 realDegrees = aDegrees;
        
        // Convert to positive values
        if ( intDegrees < 0 )
            {
            intDegrees = -intDegrees;
            realDegrees = -realDegrees;
            }

        // Minutes
        TReal64 realMinutes = (realDegrees - intDegrees) * KSecondsInMinute;
          
        // Integer part of the minutes
        TInt intMinutes = static_cast<TInt>(realMinutes);

        // Seconds
        TReal64 realSeconds = (realMinutes - intMinutes) * KSecondsInMinute;
        TInt intSeconds = static_cast<TInt>((realMinutes - intMinutes) * KSecondsInMinute);

        // Check the sign of the result
        if ( aDegrees >= 0 )
            {
            aDegreesString.Append(KDelimPlus); 
            }
        else
            {
            aDegreesString.Append(KDelimMinus);
            }

        // Add the degrees
        TInt64 value = intDegrees;
        aDegreesString.AppendNum(value);

        // Add the separator
        aDegreesString.Append(KDelimDegree);
    
        // Add the minutes
        value = intMinutes;
        aDegreesString.AppendNum(value);

        // Add the separator
        aDegreesString.Append(KApostrophe);
        
        // Add the seconds
        value = intSeconds;
        aDegreesString.AppendNum(value);

        // Add the separator
        aDegreesString.Append(KDelimQuot);

        }
    else
        {
        // The conversion can not be done, return NaN
        aDegreesString = KNan;
        }
    }
    


void CLocationContainer::SetLocation(CLocation* aLocation)
    {

    AddRowL(*iCountryHeading, aLocation->iDescription.iCountry);
    AddRowL(*iIcaoHeading, aLocation->iDescription.iIcao);
    
    TBuf<50> degrees;
    GetDegreesString(aLocation->iDescription.iCoordinate.Latitude(), degrees);
    AddRowL(*iLatitudeHeading, degrees);

    degrees.Zero();
    GetDegreesString(aLocation->iDescription.iCoordinate.Longitude(), degrees);
    AddRowL(*iLongitudeHeading, degrees);

    iListBox->HandleItemAdditionL();
    DrawNow();

    }

TTypeUid::Ptr CLocationContainer::MopSupplyObject(TTypeUid aId)
    {
    if (iBgContext )
        {
        return MAknsControlContext::SupplyMopObject( aId, iBgContext );
        }
    return CCoeControl::MopSupplyObject(aId);
    }


void CLocationContainer::SizeChanged()
    {
    if(iBgContext)
        {
        iBgContext->SetRect(Rect());
        iListBox->SetRect(Rect());
        if ( &Window() )
            {
            iBgContext->SetParentPos( PositionRelativeToScreen() );
            }
        }   
    }

void CLocationContainer::HandleResourceChange(TInt aType)
    {
    RDebug::Print(_L("!HandleResourceChange"));
    CCoeControl::HandleResourceChange(aType);
    // ADDED FOR SCALABLE UI SUPPORT
    // *****************************
    if ( aType==KEikDynamicLayoutVariantSwitch )
        {
        TRect rect;
        AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane, rect);
        SetRect(rect);
        }
    }


TInt CLocationContainer::CountComponentControls() const
    {
    return 1;
    }


CCoeControl* CLocationContainer::ComponentControl(TInt /*aIndex*/) const
    {

    return iListBox;

    }

void CLocationContainer::Draw(const TRect& aRect) const
    {

    CWindowGc& gc = SystemGc();

    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    MAknsControlContext* cc = AknsDrawUtils::ControlContext(this);
    AknsDrawUtils::Background(skin, cc, this, gc, aRect);
    
    }

