
/*
 * Copyright Justin Armstrong 2008. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include "locationdb.h"
#include "weatherutils.h"
#include <weather.rsg>
#include "weather.hrh"
#include <f32file.h>
#include <S32FILE.H>


//--------------------------------------------



CLocationDB::CLocationDB()
    {

    }

CLocationDB::~CLocationDB()
    {

    }

TPtrC CLocationDB::GetNextElement(TDesC& aLine, TInt& aStart)
    {

    TPtrC remaining = aLine.Mid(aStart);
    TInt index = remaining.Locate((TChar)';');
    if (index != KErrNotFound) 
        {
        TPtrC element = aLine.Mid(aStart, index);
        aStart = aStart+index+1;
        return element;
        } else {
        aStart = -1;
        return TPtrC();
        }
    }


_LIT(KLocationsTable, "Locations"); 
_LIT(KCountryTable, "CountryTable"); //has only the Country column

_LIT(KLocationsCountryCol, "Country");
_LIT(KLocationsStateCol, "State");
_LIT(KLocationsPlaceNameCol, "PlaceName");
_LIT(KLocationsLatitudeCol, "Latitude");
_LIT(KLocationsLongitudeCol, "Longitude");
_LIT(KLocationsElevationCol, "Elevation");
_LIT(KLocationsIcaoCol, "ICAO");



CDesCArrayFlat* CLocationDB::GetAllCountriesL()
    {

    TPtrC country;
    TBuf<200> rowText;

    _LIT(KSelect, "SELECT ");
    _LIT(KFrom, " FROM ");
    _LIT(KOrderBy, " ORDER BY ");

    // Sql: SELECT Country FROM Locations
    //      ORDER BY Country
    TBuf<100> sqlStr;
    sqlStr.Append(KSelect);
    sqlStr.Append(KLocationsCountryCol);
    sqlStr.Append(KFrom);
    sqlStr.Append(KCountryTable);
    sqlStr.Append(KOrderBy);
    sqlStr.Append(KLocationsCountryCol);


    // Create a view on the database
    RDbView view;
    User::LeaveIfError(
        view.Prepare(iDb, TDbQuery(sqlStr), view.EReadOnly));
    User::LeaveIfError(view.EvaluateAll());

    CDbColSet* colSet = view.ColSetL();
    CleanupStack::PushL(colSet);
    
    TInt rows = Max(view.CountL(), 10);
    CDesCArrayFlat* resultArray =
        new (ELeave)CDesC16ArrayFlat(rows);
    CleanupStack::PushL(resultArray);    
    
    // Append each result row to array
    for (view.FirstL(); view.AtRow(); view.NextL())
        {
        view.GetL();
        country.Set(view.ColDes(colSet->ColNo(KLocationsCountryCol)));
        resultArray->AppendL(country);
        }

    CleanupStack::Pop(resultArray);    
    CleanupStack::PopAndDestroy(colSet);
    view.Close();


    return resultArray;

    }



CDesCArrayFlat* CLocationDB::GetStatesInCountryL(TPtrC aCountry)
    {
    TPtrC state;
    
    _LIT(KSelect, "SELECT ");
    _LIT(KFrom, " FROM ");
    _LIT(KWhere, " WHERE ");
    _LIT(KOrderBy, " ORDER BY ");
    _LIT(KEqual, " = ");
    _LIT(KSingleQuote, "'");

    // Sql: SELECT Country FROM Locations
    //      ORDER BY Country
    TBuf<100> sqlStr;
    sqlStr.Append(KSelect);
    sqlStr.Append(KLocationsStateCol);
    sqlStr.Append(KFrom);
    sqlStr.Append(KLocationsTable);
    sqlStr.Append(KWhere);
    sqlStr.Append(KLocationsCountryCol);
    sqlStr.Append(KEqual);
    sqlStr.Append(KSingleQuote); 
    sqlStr.Append(aCountry);
    sqlStr.Append(KSingleQuote);
    sqlStr.Append(KOrderBy);
    sqlStr.Append(KLocationsStateCol);


    // Create a view on the database
    RDbView view;
    User::LeaveIfError(
        view.Prepare(iDb, TDbQuery(sqlStr), view.EReadOnly));
    User::LeaveIfError(view.EvaluateAll());

    
    TInt rows = Max(view.CountL(), 10);
    CDesCArrayFlat* resultArray =
        new (ELeave)CDesC16ArrayFlat(rows);
    CleanupStack::PushL(resultArray);
    
    CDbColSet* colSet = view.ColSetL();
    CleanupStack::PushL(colSet);

    // Append each result row to array
    for (view.FirstL(); view.AtRow(); view.NextL())
        {
        view.GetL();
        state.Set(view.ColDes(colSet->ColNo(KLocationsStateCol)));
        if (state.Length() > 0)
            {
            TInt pos;
            if (resultArray->Find(state, pos) != 0)
                {
                resultArray->AppendL(state);
                }
            }
        }
    
    CleanupStack::PopAndDestroy(colSet);
    view.Close();
    CleanupStack::Pop(resultArray);

    return resultArray;

    }



RPointerArray<TLocationDescription>* CLocationDB::GetLocationsInCountryL(TPtrC aCountry, TPtrC aState)
    {

    _LIT(KSelectStar, "SELECT *");
    _LIT(KFrom, " FROM ");
    _LIT(KWhere, " WHERE ");
    _LIT(KOrderBy, " ORDER BY ");
    _LIT(KAnd, " AND ");
    _LIT(KEqual, " = ");
    _LIT(KSingleQuote, "'");

    // Sql: SELECT Country FROM Locations
    //      ORDER BY Country
    TBuf<100> sqlStr;
    sqlStr.Append(KSelectStar);
    sqlStr.Append(KFrom);
    sqlStr.Append(KLocationsTable);
    sqlStr.Append(KWhere);
    sqlStr.Append(KLocationsCountryCol);
    sqlStr.Append(KEqual);
    sqlStr.Append(KSingleQuote); 
    sqlStr.Append(aCountry);
    sqlStr.Append(KSingleQuote);
    if (aState.Length() > 0)
        {
        sqlStr.Append(KAnd);
        sqlStr.Append(KLocationsStateCol);
        sqlStr.Append(KEqual);
        sqlStr.Append(KSingleQuote); 
        sqlStr.Append(aState);
        sqlStr.Append(KSingleQuote);
        }
    sqlStr.Append(KOrderBy);
    sqlStr.Append(KLocationsPlaceNameCol);

    // Create a view on the database
    RDbView view;
    User::LeaveIfError(
        view.Prepare(iDb, TDbQuery(sqlStr), view.EReadOnly));
    User::LeaveIfError(view.EvaluateAll());

    CDbColSet* colSet = view.ColSetL();
    CleanupStack::PushL(colSet);
    TInt rows = Max(view.CountL(), 10);
    
    RPointerArray<TLocationDescription>* locationArray =
        new(ELeave)RPointerArray<TLocationDescription>(rows);
    CleanupStack::PushL(locationArray);

    // Append each result row to array
    for (view.FirstL(); view.AtRow(); view.NextL())
        {
        view.GetL();

        TLocationDescription* locDesc = new (ELeave) TLocationDescription;
        locDesc->iPlaceName = view.ColDes(colSet->ColNo(KLocationsPlaceNameCol));
        locDesc->iIcao = view.ColDes(colSet->ColNo(KLocationsIcaoCol));
        locDesc->iCountry = view.ColDes(colSet->ColNo(KLocationsCountryCol));
        locDesc->iState = view.ColDes(colSet->ColNo(KLocationsStateCol));
        locDesc->iCoordinate = TCoordinate(view.ColReal64(colSet->ColNo(KLocationsLatitudeCol)), 
                                            view.ColReal64(colSet->ColNo(KLocationsLongitudeCol)));
        
        locDesc->iDistance = 0;    //not applicable here
        locationArray->Append(locDesc);

        }
    CleanupStack::Pop(locationArray);
    CleanupStack::PopAndDestroy(colSet);
    view.Close();


    return locationArray;

    }


bool CLocationDB::GetLocationDescriptionByIcao(const TPtrC aIcao, TLocationDescription* aLocDesc)
    {
    _LIT(KSelectStar, "SELECT *");
    _LIT(KFrom, " FROM ");
    _LIT(KWhere, " WHERE ");
    _LIT(KEqual, " = ");
    _LIT(KSingleQuote, "'");
    
    // Sql: SELECT Country FROM Locations
    //      ORDER BY Country
    TBuf<100> sqlStr;
    sqlStr.Append(KSelectStar);
    sqlStr.Append(KFrom);
    sqlStr.Append(KLocationsTable);
    sqlStr.Append(KWhere);
    sqlStr.Append(KLocationsIcaoCol);
    sqlStr.Append(KEqual);
    sqlStr.Append(KSingleQuote); 
    sqlStr.Append(aIcao);
    sqlStr.Append(KSingleQuote);
    
    // Create a view on the database
    RDbView view;
    User::LeaveIfError(
        view.Prepare(iDb, TDbQuery(sqlStr), view.EReadOnly));
    User::LeaveIfError(view.EvaluateAll());
    
    CDbColSet* colSet = view.ColSetL();
    CleanupStack::PushL(colSet);
    
    bool found = false;
    // Append each result row to array
    for (view.FirstL(); view.AtRow(); view.NextL())
        {
        found = true;
        view.GetL();
        aLocDesc->iPlaceName = view.ColDes(colSet->ColNo(KLocationsPlaceNameCol));
        aLocDesc->iIcao = view.ColDes(colSet->ColNo(KLocationsIcaoCol));
        aLocDesc->iCountry = view.ColDes(colSet->ColNo(KLocationsCountryCol));
        aLocDesc->iState = view.ColDes(colSet->ColNo(KLocationsStateCol));
        aLocDesc->iCoordinate = TCoordinate(view.ColReal64(colSet->ColNo(KLocationsLatitudeCol)), 
                                            view.ColReal64(colSet->ColNo(KLocationsLongitudeCol)));
        
        aLocDesc->iDistance = 0;    //not applicable here
    
        }
    
    CleanupStack::PopAndDestroy(colSet);
    view.Close();

    return found;

    }

TReal64 AddAndClip(TReal64 aVal, TReal64 aAbsMax, TReal64 aOffset)
    {
    TReal64 result = aVal + aOffset;
    TInt sign;
    if (result > 0.0) sign = 1;
    else sign = -1;
    TReal64 absVal = result * sign;
    if (absVal > aAbsMax)
        {
        result = aAbsMax - (absVal - aAbsMax);
        sign = sign * -1;
        }
    else
        {
        result = absVal;
        }
    result *= sign;
    return result;
    
    }

RPointerArray<TLocationDescription>* CLocationDB::FindNearbyLocationsL(TReal64 aLatitude, TReal64 aLongitude)
    {
    TReal64 longitudeMin, longitudeMax;
    TReal64 latitudeMin, latitudeMax;
    
    TReal64 longitudeA, longitudeB;
    longitudeA = AddAndClip(aLongitude, 180.0, -1.0);
    longitudeB = AddAndClip(aLongitude, 180.0, +1.0);
    if (longitudeA > longitudeB)
        {
        longitudeMax = longitudeA;
        longitudeMin = longitudeB;
        }
    else
        {
        longitudeMin = longitudeA;
        longitudeMax = longitudeB;
        }
    
    TReal64 latitudeA, latitudeB;
    latitudeA = AddAndClip(aLatitude, 90.0, -1.0);
    latitudeB = AddAndClip(aLatitude, 90.0, +1.0);
    if (latitudeA > latitudeB)
        {
        latitudeMax = latitudeA;
        latitudeMin = latitudeB;
        }
    else
        {
        latitudeMin = latitudeA;
        latitudeMax = latitudeB;
        }
    

    TBuf<100> rowText;

    _LIT(KSelectStar, "SELECT *");
    _LIT(KFrom, " FROM ");
    _LIT(KWhere, " WHERE ");

    // Sql: SELECT Country FROM Locations
    //      ORDER BY Country
    TBuf<200> sqlStr;
    sqlStr.Append(KSelectStar);
    sqlStr.Append(KFrom);
    sqlStr.Append(KLocationsTable);
    sqlStr.Append(KWhere);
    sqlStr.Append(KLocationsLatitudeCol);
    sqlStr.Append(_L(" > "));
    sqlStr.AppendFormat(_L(" %.2f "), latitudeMin); 
    sqlStr.Append(_L(" AND "));
    sqlStr.Append(KLocationsLatitudeCol);
    sqlStr.Append(_L(" < "));
    sqlStr.AppendFormat(_L(" %.2f "), latitudeMax);
    sqlStr.Append(_L(" AND "));
    sqlStr.Append(KLocationsLongitudeCol);
    sqlStr.Append(_L(" > "));
    sqlStr.AppendFormat(_L(" %.2f "), longitudeMin); 
    sqlStr.Append(_L(" AND "));
    sqlStr.Append(KLocationsLongitudeCol);
    sqlStr.Append(_L(" < "));
    sqlStr.AppendFormat(_L(" %.2f "), longitudeMax); 

   
    RPointerArray<TLocationDescription>* locationArray =
        new(ELeave)RPointerArray<TLocationDescription>(4);
    CleanupStack::PushL(locationArray);

    // Create a view on the database
    RDbView view;
    User::LeaveIfError(
        view.Prepare(iDb, TDbQuery(sqlStr), view.EReadOnly));
    User::LeaveIfError(view.EvaluateAll());

    CDbColSet* colSet = view.ColSetL();
    CleanupStack::PushL(colSet);

    TCoordinate currentPos(aLatitude, aLongitude);

    
    // Append each result row to array
    for (view.FirstL(); view.AtRow(); view.NextL())
        {
        rowText.Zero();
        view.GetL();
        TCoordinate stationPos(view.ColReal64(colSet->ColNo(KLocationsLatitudeCol)), 
                               view.ColReal64(colSet->ColNo(KLocationsLongitudeCol)));
        TReal32 distance;
        stationPos.Distance(currentPos, distance);
        TLocationDescription* locDesc = new (ELeave) TLocationDescription;
        locDesc->iPlaceName = view.ColDes(colSet->ColNo(KLocationsPlaceNameCol));
        locDesc->iIcao = view.ColDes(colSet->ColNo(KLocationsIcaoCol));
        locDesc->iCountry = view.ColDes(colSet->ColNo(KLocationsCountryCol));
        locDesc->iState = view.ColDes(colSet->ColNo(KLocationsStateCol));
        locDesc->iCoordinate = TCoordinate(view.ColReal64(colSet->ColNo(KLocationsLatitudeCol)), 
                                           view.ColReal64(colSet->ColNo(KLocationsLongitudeCol)));
        locDesc->iDistance = distance;
        locationArray->Append(locDesc);
        }
    if (!view.IsEmptyL())
        {
        TLinearOrder<TLocationDescription> byDistance( TLocationDescription::CompareByDistance );
        locationArray->Sort( byDistance );
        }
    
    //trim it down to 10 elements
    while (locationArray->Count() > 10)
        {
        TUint last = locationArray->Count()-1;
        TLocationDescription* locDesc = (*locationArray)[last];
        locationArray->Remove(last);
        delete locDesc;
        }
    
    CleanupStack::PopAndDestroy(colSet);
    view.Close();
    CleanupStack::Pop(locationArray);

    return locationArray;

    }


void CLocationDB::OpenLocationsDbL(TFileName& aDbFileName)
    {
    Close();

    User::LeaveIfError(iFsSession.Connect());
     TRAPD(error, 
         iFileStore = CPermanentFileStore::OpenL(iFsSession, aDbFileName, 
             EFileRead|EFileWrite);
         iFileStore->SetTypeL(iFileStore->Layout());/* Set file store type*/
         iDb.OpenL(iFileStore,iFileStore->Root())
         );
    if(error!=KErrNone)
        {
        return;
        }
     
     iOpen = ETrue;
    }



void CLocationDB::CreateTablesL()
    {

    // Specify columns for locations table
    TDbCol countryCol(KLocationsCountryCol, EDbColText);
    TDbCol placeNameCol(KLocationsPlaceNameCol, EDbColText, 100);
    TDbCol latitudeCol(KLocationsLatitudeCol, EDbColReal64);
    TDbCol longitudeCol(KLocationsLongitudeCol, EDbColReal64);
    TDbCol stateCol(KLocationsStateCol, EDbColText);
    TDbCol icaoCol(KLocationsIcaoCol, EDbColText);
    

    // Add the columns to column set
    CDbColSet* locationsColSet = CDbColSet::NewLC();
    locationsColSet->AddL(countryCol);
    locationsColSet->AddL(placeNameCol);
    locationsColSet->AddL(latitudeCol);
    locationsColSet->AddL(longitudeCol);
    locationsColSet->AddL(stateCol);
    locationsColSet->AddL(icaoCol);    
    User::LeaveIfError(iDb.CreateTable(KLocationsTable,
        *locationsColSet));
    CleanupStack::PopAndDestroy(locationsColSet);
    
    //create table that just holds all the country names
    CDbColSet* countryColSet = CDbColSet::NewLC();
    countryColSet->AddL(countryCol);
    User::LeaveIfError(iDb.CreateTable(KCountryTable,
        *countryColSet));
    CleanupStack::PopAndDestroy(countryColSet);
    
    }


_LIT(KCountryIndexName, "CountryIndex"); 
_LIT(KPlaceNameIndexName, "PlaceNameIndex"); 

void CLocationDB::CreateIndicesL()
    {
    TDbKeyCol countryCol(KLocationsCountryCol);
    CDbKey* index;
    
    TDbKeyCol placeNameCol(KLocationsPlaceNameCol);
    index = CDbKey::NewLC();
    index->AddL(placeNameCol);
    User::LeaveIfError(iDb.CreateIndex(
            KPlaceNameIndexName, KLocationsTable, *index));
    CleanupStack::PopAndDestroy(index);
    
    //country index in country table

    index = CDbKey::NewLC();   // create index key set
    index->AddL(countryCol);
    User::LeaveIfError(iDb.CreateIndex(
        KCountryIndexName, KCountryTable, *index));
    CleanupStack::PopAndDestroy(index);
    
    
    }

TReal64 CLocationDB::ParseLatitude(TPtrC string)
    {
    TInt dashPos = string.Find(_L("-"));
    if (dashPos == KErrNotFound)
        {
        return 0.0;
        }
    TPtrC degreeString = string.Left(dashPos);
    TLex degLex(degreeString);
    TInt degrees;
    if (degLex.Val(degrees) != KErrNone)
        {
        return 0.0;
        }
    TPtrC minuteString = string.Mid(dashPos+1, 2);
    TLex minLex(minuteString);
    TInt minutes;
    if (minLex.Val(minutes) != KErrNone)
        {
        return 0.0;
        }
    //look for seconds - this is optional
    TInt seconds = 0;
    TPtrC remaining = string.Mid(dashPos+2, string.Length()-(dashPos+2));   
    dashPos = remaining.Find(_L("-"));
    if (dashPos != KErrNotFound)
        {
        TPtrC secondString = remaining.Mid(dashPos+1, 2);
        TLex secLex(secondString);
        if (secLex.Val(seconds) != KErrNone)
            {
            seconds = 0;
            }
        }
        
    TReal64 latitude = (TReal64)degrees + (TReal64)minutes/60.0 + (TReal64)seconds/3600.0;
    TPtrC directionString = string.Right(1);
    if (directionString.Compare(_L("S")) == 0) 
        {
        latitude *= -1;
        }
    return latitude;
    }

TReal64 CLocationDB::ParseLongitude(TPtrC string)
    {
    
    TInt dashPos = string.Find(_L("-"));
    if (dashPos == KErrNotFound)
        {
        return 0.0;
        }
    TPtrC degreeString = string.Left(dashPos);
    TLex degLex(degreeString);
    TInt degrees;
    if (degLex.Val(degrees) != KErrNone)
        {
        return 0.0;
        }
    TPtrC minuteString = string.Mid(dashPos+1, 2);
    TLex minLex(minuteString);
    TInt minutes;
    if (minLex.Val(minutes) != KErrNone)
        {
        return 0.0;
        }
    //look for seconds - this is optional
    TInt seconds = 0;
    TPtrC remaining = string.Mid(dashPos+2, string.Length()-(dashPos+2));   
    dashPos = remaining.Find(_L("-"));
    if (dashPos != KErrNotFound)
        {
        TPtrC secondString = remaining.Mid(dashPos+1, 2);
        TLex secLex(secondString);
        if (secLex.Val(seconds) != KErrNone)
            {
            seconds = 0;
            }
        }
    
    TReal64 longitude = (TReal64)degrees + (TReal64)minutes/60.0 + (TReal64)seconds/3600.0;
    TPtrC directionString = string.Right(1);
    if (directionString.Compare(_L("W")) == 0) 
        {
        longitude *= -1;
        }
    return longitude;
    }

/*
 * Keyed by Location Indicator
 * All stations that have a location indicator assigned by The ICAO are available in this file. The ASCII file contains The following fields in The order specified, delimited by semicolons. Fields that are empty will be coded as no characters (adjacent delimiters) except where otherwise noted.
 * 
 * ICAO Location
 * Indicator     4 alphanumeric characters.
 * Block Number     2 digits representing The WMO-assigned block. Not all stations in The file have an assigned block number. The value "--" is used for station without an assigned number.
 * Station Number     3 digits representing The WMO-assigned station. Not all stations in The file have an assigned station number. The value "---" is used for station without an assigned number.
 * Place Name     Common name of station location.
 * State     2 character abbreviation (included for stations located in The United States only).
 * Country Name     Country name is ISO short English form.
 * WMO Region     Digits 1 through 6 representing The corresponding WMO region, 7 stands for The WMO Antarctic region.
 * Station Latitude     DD-MM-SSH where DD is degrees, MM is minutes, SS is seconds and H is N for northern hemisphere or S for southern hemisphere. The seconds value is omitted for those stations where The seconds value is unknown.
 * Station Longitude     DDD-MM-SSH where DDD is degrees, MM is minutes, SS is seconds and H is E for eastern hemisphere or W for western hemisphere. The seconds value is omitted for those stations where The seconds value is unknown.
 * Upper Air Latitude     DD-MM-SSH where DD is degrees, MM is minutes, SS is seconds and H is N for northern hemisphere or S for southern hemisphere. The seconds value is omitted for those stations where The seconds value is unknown. 
 * Upper Air Longitude     DDD-MM-SSH where DDD is degrees, MM is minutes, SS is seconds and H is E for eastern hemisphere or W for western hemisphere. The seconds value is omitted for those stations where The seconds value is unknown.
 * Station Elevation (Ha)     The station elevation in meters. Value is omitted if unknown.
 * Upper Air Elevation (Hp)     The upper air elevation in meters. Value is omitted if unknown.
 * RBSN indicator     P if station is defined by The WMO as belonging to The Regional Basic Synoptic Network, omitted otherwise.
 * 
 * */
 
void CLocationDB::ParseLocationsFileL(RFile& aLocationsFile)
    {

    TBuf<256> buf;
    TFileText fileReader;
    fileReader.Set(aLocationsFile);

    RDbTable locationsTable;
    RDbTable countryTable;
    TInt err = locationsTable.Open(iDb, KLocationsTable, locationsTable.EUpdatable);    
    if(err!=KErrNone)
    {
        RDebug::Print(_L("\error opening table %d"), err);
    }
    err = countryTable.Open(iDb, KCountryTable, countryTable.EUpdatable);
    err = countryTable.SetIndex(KCountryIndexName);
    
    CleanupClosePushL(locationsTable);
    CleanupClosePushL(countryTable);
    
    locationsTable.Reset();
    countryTable.Reset();
    
    TInt retVal;
    do {
        retVal = fileReader.Read(buf);
        if (retVal == KErrNone) 
            {
            RDebug::Print(_L("\nreport=<%S>"), &buf);
            
            TPtrC element;
            TInt index = 0;
            /*
            do 
                {
                element.Set(GetNextElement(buf, index));
                } while (index != -1);
            }
            */
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // icao code
            TPtrC icaoCode = element; 
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // block number
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // station number            
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // place name
            TPtrC placeName = element;
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // state
            TPtrC state = element;
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // country      
            TPtrC country = element;            
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // WMO Region      
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // latitude
            TReal64 latitude = ParseLatitude(element);
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // longitude
            TReal64 longitude = ParseLongitude(element);
            element.Set(GetNextElement(buf, index)); if (index == -1) break; // elevation
            TPtrC elevation = element;   
            
            RDebug::Print(_L("parsed: %S, %S, %S, %S, %f, %f, %S"),
                    &icaoCode, &placeName, &state, &country, latitude, longitude, &elevation);
            
            CDbColSet* colSet = locationsTable.ColSetL();
            CleanupStack::PushL(colSet);

            locationsTable.InsertL(); // Insert empty row
            // Set the author and title for the new row
            locationsTable.SetColL(colSet->ColNo(KLocationsIcaoCol), icaoCode);
            locationsTable.SetColL(colSet->ColNo(KLocationsPlaceNameCol), placeName);
            locationsTable.SetColL(colSet->ColNo(KLocationsStateCol), state);            
            locationsTable.SetColL(colSet->ColNo(KLocationsCountryCol), country);            
            locationsTable.SetColL(colSet->ColNo(KLocationsLatitudeCol), latitude);        
            locationsTable.SetColL(colSet->ColNo(KLocationsLongitudeCol), longitude);                    
            //table.SetColL(colSet->ColNo(KLocationsElevationCol), elevation);         

            locationsTable.PutL(); // Complete insertion       
            CleanupStack::PopAndDestroy(colSet);
            
            //----
            //add country name to CountryTable if it has not been added already
            //----
            
            colSet = countryTable.ColSetL();
            CleanupStack::PushL(colSet);
            TDbSeekKey seekKey(country); // Initialize one-column seek key

            // Search the index for aTitle
            TBool found = false;
            TRAP(err, found = countryTable.SeekL(seekKey));
            if(!found) 
                {
                CDbColSet* insertCountryColSet = locationsTable.ColSetL();
                CleanupStack::PushL(insertCountryColSet);
                countryTable.InsertL(); // Insert empty row
                countryTable.SetColL(insertCountryColSet->ColNo(KLocationsCountryCol), country); 
                countryTable.PutL(); // Complete insertion       
                CleanupStack::PopAndDestroy(insertCountryColSet);
                }
            CleanupStack::PopAndDestroy(colSet);
            
            }
        } while(retVal == KErrNone);
    
        CleanupStack::Pop(); // countryTable
        countryTable.Close();       
    
        CleanupStack::Pop(); // locationsTable
        locationsTable.Close();

    }
    
void CLocationDB::CreateLocationsDbL(TFileName& aDbFileName)
    {
    Close();
    
    User::LeaveIfError(iFsSession.Connect());
    TRAPD(error,
        iFileStore = CPermanentFileStore::ReplaceL(iFsSession, aDbFileName, 
            EFileRead|EFileWrite);
        iFileStore->SetTypeL(iFileStore->Layout());// Set file store type
        TStreamId id = iDb.CreateL(iFileStore);// Create stream object
        iFileStore->SetRootL(id);// Keep database id as root of store
        iFileStore->CommitL();// Complete creation by commiting
        // Create Book tables and indexes    
          CreateTablesL();
          CreateIndicesL();
        );
        
    iOpen = ETrue;    
    }

void CLocationDB::CreateLocationsDbFromLocationsFileL(TFileName& aDbFileName, RFile& aLocationsFile)
    {
    CreateLocationsDbL(aDbFileName);
    ParseLocationsFileL(aLocationsFile);
    iDb.CompressL(*iFileStore, iFileStore->Root());
    iDb.Compact();
    iDb.Close();
    }


TInt CLocationDB::Close()
    {
    iDb.Close();
    if(iFileStore)
        {
        delete iFileStore;
        iFileStore = NULL;
        }
    iFsSession.Close();
    iOpen = EFalse;
    return KErrNone;
    }



void CLocationDB::ConstructL(CCoeEnv* /*aCoeEnv*/)
    {



    }
