/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <akniconutils.h> 
#include "thermometer.h"
#include "weatherutils.h"


#define THERM_HEIGHT 30 //pixels
#define THERM_MIN_TEMP -10  //celsius
#define THERM_PIXELS_PER_DEGREE 2
#define THERM_MAX_TEMP (THERM_MIN_TEMP+(THERM_HEIGHT*THERM_PIXELS_PER_DEGREE))


void _DrawTemperatureOnBitmap(TInt aTemperature, CGraphicsContext* bitmapContext, TBool aIsMask)
    {

    TRect textRect(TPoint(0, 0), TSize(60, 58));

    //CFont* font;
    //_LIT(KTxtFontName,"LatinBold12");
    //TFontSpec fontSpec(KTxtFontName(),textRect.Height());
    //bitmapDevice->GetNearestFontInPixels(font, fontSpec);
    
    const CFont* font = CEikonEnv::Static()->LegendFont();
    //const CFont* font = AknLayoutUtils::FontFromId(EAknLogicalFontSecondaryFont, NULL);
    bitmapContext->UseFont(font);
    TRgb colour;
    if (aIsMask)
        {
        colour = KRgbBlack; 
        }
    else 
        {
        MAknsSkinInstance* skin = AknsUtils::SkinInstance();
        AknsUtils::GetCachedColor(skin, colour, KAknsIIDQsnTextColors, EAknsCIQsnTextColorsCG6);
        }
    
    bitmapContext->SetPenColor(colour);
    
    bitmapContext->SetPenStyle(CGraphicsContext::ESolidPen);
    bitmapContext->SetDrawMode(CGraphicsContext::EDrawModePEN);
    bitmapContext->SetBrushStyle(CGraphicsContext::ESolidBrush);
    bitmapContext->SetBrushColor(0x00ff00);

    TBuf<6> tempString;
    MakeTempString(tempString, aTemperature);

    bitmapContext->DrawText(tempString, TPoint(20, 20)); //was 4, 2
    //bitmapContext->DrawText(tempString, textRect, font->AscentInPixels()+6, CGraphicsContext::ERight);

    bitmapContext->DiscardFont();


    }

void CThermometer::DrawTemperatureL(TInt aTemperature, CFbsBitmap*& aBitmap, CFbsBitmap*& aMaskBitmap)
    {

    TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();
    TParsePtr parser(app);
    TFileName mbmFile = parser.Drive();
    mbmFile.Append(_L("\\resource\\apps\\weather.mbm"));
    CFbsBitmap* thermBitmap;
    CFbsBitmap* thermMask;
    //AknIconUtils::CreateIconL(thermBitmap, thermMask, mbmFile, EMbmWeatherSvgtherm, EMbmWeatherSvgtherm_mask);        
    AknIconUtils::CreateIconL(thermBitmap, thermMask, mbmFile, EMbmWeatherTherm, EMbmWeatherTherm_mask);   
    //AknIconUtils::CreateIconL(thermBitmap, thermMask, mbmFile, EMbmWeatherQgn_menu_weather, EMbmWeatherQgn_menu_weather_mask);    
    //AknIconUtils::CreateIconL(thermBitmap, thermMask, mbmFile, EMbmWeatherSvgtherm, EMbmWeatherSvgtherm);  
    
    RDebug::Print(_L("seems to have loaded okay"));
    
    //AknIconUtils::ExcludeFromCache(thermBitmap);
    //AknIconUtils::ExcludeFromCache(thermMask); 
    
    TSize thermSize(48, 48);
    //AknIconUtils::SetSize(thermBitmap, thermSize, EAspectRatioPreserved);   
    //AknIconUtils::SetSize(thermMask, thermSize, EAspectRatioPreserved);   
    
    //TSize size;
    //AknLayoutUtils::LayoutMetricsSize(AknLayoutUtils::EContextPane, size); 
    //AknIconUtils::SetSize(thermBitmap, size, EAspectRatioPreserved);   
    //AknIconUtils::SetSize(thermMask, size, EAspectRatioPreserved);    
    
    aBitmap = new (ELeave) CFbsBitmap();
    CleanupStack::PushL(aBitmap);
    aBitmap->Create(thermSize, EColor16M);
    //TInt result = aBitmap->Load(mbmFile, EMbmWeatherTherm);
    //User::LeaveIfError(aBitmap->Load(mbmFile, EMbmWeatherTherm));
    //AknIconUtils::SetSize(aBitmap, size, EAspectRatioPreserved);

    aMaskBitmap = new (ELeave) CFbsBitmap();
    CleanupStack::PushL(aMaskBitmap);
    aMaskBitmap->Create(thermSize, EGray2);
    //User::LeaveIfError(aMaskBitmap->Load(mbmFile, EMbmWeatherTherm_mask));
    //AknIconUtils::SetSize(aMaskBitmap, size, EAspectRatioPreserved);    


    CFbsBitGc* bitmapContext=NULL;
    CFbsBitmapDevice* bitmapDevice = CFbsBitmapDevice::NewL(aBitmap);
    CleanupStack::PushL(bitmapDevice);
    User::LeaveIfError(bitmapDevice->CreateContext(bitmapContext));
    CleanupStack::PushL(bitmapContext);

    CFbsBitGc* maskContext=NULL;
    CFbsBitmapDevice* maskDevice = CFbsBitmapDevice::NewL(aMaskBitmap);
    CleanupStack::PushL(maskDevice);
    User::LeaveIfError(maskDevice->CreateContext(maskContext));
    CleanupStack::PushL(maskContext);

    //draw the temp, in both the bitmap and the mask
    bitmapContext->Clear();
    maskContext->Clear();
    bitmapContext->SetPenStyle(CGraphicsContext::ESolidPen);

    //TRect bitmapRect(TPoint(0, 0), size/*thermBitmap->SizeInPixels()*/);
    //bitmapContext->DrawBitmapMasked(TRect(TPoint(0,44), thermSize), thermBitmap, bitmapRect, thermMask, FALSE);
    //bitmapContext->DrawBitmap(TPoint(0, 44), thermBitmap);
    //bitmapContext->DrawBitmap(bitmapRect, thermBitmap);
    //maskContext->DrawBitmap(bitmapRect, thermMask);
    bitmapContext->BitBlt(TPoint(0, 0), thermBitmap);
    maskContext->BitBlt(TPoint(0, 0), thermMask);
    
    _DrawTemperatureOnBitmap(aTemperature, bitmapContext, EFalse);
    _DrawTemperatureOnBitmap(aTemperature, maskContext, ETrue);


    //draw the mercury

    TInt mercuryHeight;

    if (aTemperature < THERM_MIN_TEMP)
        mercuryHeight = 0;
    else if (aTemperature > THERM_MAX_TEMP)
        mercuryHeight = THERM_HEIGHT;
    else mercuryHeight = (aTemperature - THERM_MIN_TEMP)/THERM_PIXELS_PER_DEGREE;

    TPoint fromPt(5,31);
    TPoint endPt(5,31-mercuryHeight);
    bitmapContext->SetPenColor(KRgbRed);
    bitmapContext->DrawLine(fromPt, endPt);

    delete thermBitmap;
    delete thermMask;
    
    CleanupStack::PopAndDestroy();
    CleanupStack::PopAndDestroy();

    CleanupStack::PopAndDestroy();
    CleanupStack::PopAndDestroy();

    CleanupStack::Pop();
    CleanupStack::Pop();
    

    }

