/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

// INCLUDE FILES
#include "weatherappui.h"
#include "weathercontainer.h" 
#include <weather.rsg>
#include "weather.hrh"
#include "utf.h"

#include "aboutdialog.h"

#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <aknglobalnote.h> 
#include <aknmessagequerydialog.h> 

//#include <intconninit.h> 
#include <commdb.h>
#include <apaccesspointitem.h> 
#include <apdatahandler.h> 
#include <apselect.h> 
#include <BAUTILS.H>
#include <lbs.h>

#include "locationviewcontroller.h"


// copied from Symbian Knowledgebase FAQ-0535
TAny* TKeyArrayPtr::At(TInt aIndex) const
{
    if (aIndex==KIndexPtr)
        return *(TUint8**)iPtr+iKeyOffset;

    return *(TUint8**)iBase->Ptr(aIndex*sizeof(TUint8**)).Ptr()+iKeyOffset;
}




//---------------------------------

CGlobals* globals;

_LIT(KEmptyLocations, "location array empty");

void CWeatherAppUi::ConstructL()
    {
    RDebug::Print(_L("CWeatherAppUi::ConstructL 1!\n"));
    BaseConstructL(EAknEnableSkin | EAppOrientationAutomatic);
    RDebug::Print(_L("CWeatherAppUi::ConstructL 2!\n"));
    
    iPositionRequestor = CLbsPositionRequestor::NewL(*this);
    
 /*   
    CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();
    globalNote->ShowNoteL(EAknGlobalInformationNote, _L("CWeatherAppUi::ConstructL"));
    CleanupStack::PopAndDestroy();
*/    

    iWeatherView = CWeatherViewController::NewL();
    iLocationView = CLocationViewController::NewL();

    // Transfer ownership to base class
    AddViewL(iWeatherView);
    AddViewL(iLocationView);
    SetDefaultViewL(*iWeatherView);
    
    
    globals = new (ELeave) CGlobals;

    GLOBALS->iParser = CWeatherParser::NewL(iCoeEnv);
    
    iTxtFormatDelete = iCoeEnv->AllocReadResourceL(R_TEXT_FORMAT_DELETE);
    iTxtDeleteLocation = iCoeEnv->AllocReadResourceL(R_TEXT_DELETE_LOCATION);
    iTxtModifyLocation = iCoeEnv->AllocReadResourceL(R_TEXT_MODIFY_LOCATION);

    iHttpEngine = CHttpEngine::NewL(*this);

    LoadLocationsL();        
    LoadUserSettingsL();
    
    DisplayLocationL(); 

    }
                               
const TUid KUidLocationsIni= { 0x00000001 };



void CWeatherAppUi::LoadLocationsL()
    {

    RFs fs;
    User::LeaveIfError(fs.Connect());
    TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();    
    TParsePtr parser(app);    
    
    TFileName privatePath;
    fs.PrivatePath(privatePath);
#ifdef __WINS__   
    TFileName dbFileName = _L("C:");
#else
    TFileName dbFileName = parser.Drive();
#endif    
    dbFileName.Append(privatePath);    
    dbFileName.Append(_L("weatherlocationsdb.dat"));
    
    //dbFileName.Append(_L("\\Data\\Documents\\weatherlocationsdb.dat"));
    //dbFileName.Append(_L("\\resource\\apps\\weatherlocationsdb.dat"));
    /*
        {
        CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();
        globalNote->ShowNoteL(EAknGlobalInformationNote, _L("about to open weatherlocationsdb"));
        CleanupStack::PopAndDestroy();
        }
*/
    BaflUtils::EnsurePathExistsL(fs, dbFileName);
    if(!BaflUtils::FileExists(fs, dbFileName))
        {
        TFileName locationTextFileName;
        locationTextFileName = parser.Drive();
        locationTextFileName.Append(_L("\\resource\\apps\\weatherlocations.txt"));

        RFile locationTextFile; 
        TInt error = locationTextFile.Open(fs, locationTextFileName, EFileRead);

        iLocationsDB.CreateLocationsDbFromLocationsFileL(dbFileName, locationTextFile);
        locationTextFile.Close();
        }

    iLocationsDB.OpenLocationsDbL(dbFileName);
   
    fs.Close();
    
    
    CDictionaryStore* iniFile = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());
    TBool found = iniFile->IsPresentL(KUidLocationsIni);

    iLocationArray = new (ELeave) CArrayPtrFlat<CLocation> (4); 


    if (!found)
        {
        
        RDebug::Print(_L("no locations ini file"));
        }
    else
        {
        RDebug::Print(_L("found locations ini file"));

        RDictionaryReadStream readStream;
        readStream.OpenLC (*iniFile, KUidLocationsIni);
        TInt size = readStream.ReadInt16L();
        
        TInt i;
        for (i = 0; i < size; i++)
            {
            CLocation* location = new (ELeave) CLocation();
            CleanupStack::PushL(location);
            location->ConstructFromStreamL(&iLocationsDB, readStream);
            iLocationArray->AppendL(location);
            CleanupStack::Pop();
            }
        CleanupStack::PopAndDestroy(); 
        }

    CleanupStack::PopAndDestroy(); 

    TKeyArrayPtr locNameKey(_FOFF(CLocation,iDescription.iPlaceName), ECmpFolded);
    iLocationArray->Sort(locNameKey);

    }


const TUid KUidUserSettingsIni= { 0x00000002 };
const TUid KUidUserSettings2Ini= { 0x00000003 };


void CWeatherAppUi::LoadUserSettingsL()
    {

    CDictionaryStore* iniFile = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());
    TBool found = iniFile->IsPresentL(KUidUserSettingsIni);
    if (!found)
        {
        RDebug::Print(_L("no user settings ini file"));

        GLOBALS->iSettings.iTempUnits = ECentigrade;
        GLOBALS->iSettings.iSpeedUnits = EKnots;
        GLOBALS->iSettings.iDistanceUnits = EKilometres;
        GLOBALS->iSettings.iDetailedCloudReports = ECloudDetailed;   
        GLOBALS->iSettings.iUseProxy = EFalse;
        GLOBALS->iSettings.iProxyPort = 80;        
        }
    else
        {
        RDebug::Print(_L("found user settings ini file"));

        RDictionaryReadStream readStream;
        readStream.OpenLC (*iniFile, KUidUserSettingsIni);

        GLOBALS->iSettings.iTempUnits = readStream.ReadInt16L();
        GLOBALS->iSettings.iSpeedUnits = readStream.ReadInt16L();
        GLOBALS->iSettings.iDistanceUnits = readStream.ReadInt16L();
        GLOBALS->iSettings.iDetailedCloudReports = readStream.ReadInt16L();   

        GLOBALS->iSettings.iUseProxy = readStream.ReadInt16L();
        GLOBALS->iSettings.iProxyIpAddress.SetAddress(readStream.ReadInt32L());

        iLastLocationIndex  = readStream.ReadInt16L();        

        if ((iLastLocationIndex > -1) && (iLastLocationIndex < iLocationArray->Count()))
            {
            iCurrentLocation = iLocationArray->At(iLastLocationIndex);
            }

        GLOBALS->iSettings.iProxyPort = readStream.ReadInt16L();
        //RDebug::Print(_L("!! proxy port=%d"), GLOBALS->iSettings.iProxyPort);
                
        //for future expansion...
        TInt32 dummyInt;
        dummyInt = readStream.ReadInt32L();
        dummyInt = readStream.ReadInt32L();
        dummyInt = readStream.ReadInt32L();
        TBuf<40> dummyString;
        readStream >> dummyString;
        readStream >> dummyString;

        CleanupStack::PopAndDestroy(); 
        }
    CleanupStack::PopAndDestroy(); 
    
    }


void CWeatherAppUi::SaveLocationsL()
    {

    CDictionaryStore* store = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());
    RDictionaryWriteStream writeStream;
    writeStream.AssignLC(*store, KUidLocationsIni);


    TInt size = iLocationArray->Count();
    writeStream.WriteInt16L(size);
    TInt i;
    for (i=0; i < size; i++)
        {
        CLocation* location = iLocationArray->At(i);
        location->SaveL(writeStream);
        }

    writeStream.CommitL();
    store->CommitL();

    CleanupStack::PopAndDestroy(); 
    CleanupStack::PopAndDestroy();

    }

void CWeatherAppUi::SaveUserSettingsL()
    {

    CDictionaryStore* store = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());

    //stream 1

    RDictionaryWriteStream writeStream;
    writeStream.AssignLC(*store, KUidUserSettingsIni);

    writeStream.WriteInt16L(GLOBALS->iSettings.iTempUnits);
    writeStream.WriteInt16L(GLOBALS->iSettings.iSpeedUnits);
    writeStream.WriteInt16L(GLOBALS->iSettings.iDistanceUnits);
    writeStream.WriteInt16L(GLOBALS->iSettings.iDetailedCloudReports);

    writeStream.WriteInt16L(GLOBALS->iSettings.iUseProxy);
    writeStream.WriteInt32L(GLOBALS->iSettings.iProxyIpAddress.Address());
    writeStream.WriteInt16L(iLastLocationIndex);

    writeStream.WriteInt16L(GLOBALS->iSettings.iProxyPort);
    //RDebug::Print(_L("!! writing proxy port=%d"), GLOBALS->iSettings.iProxyPort);

    //for future expansion
    TInt32 dummyInt = 0;
    writeStream.WriteInt32L(dummyInt);
    writeStream.WriteInt32L(dummyInt);
    writeStream.WriteInt32L(dummyInt);
    TBuf<40> dummyString;
    writeStream << dummyString;
    writeStream << dummyString;

    writeStream.CommitL();
    store->CommitL();

    CleanupStack::PopAndDestroy(); 

    CleanupStack::PopAndDestroy();

    }


// ----------------------------------------------------
// CWeatherAppUi::~CWeatherAppUi()
// Destructor
// Frees reserved resources
// ----------------------------------------------------
//
CWeatherAppUi::~CWeatherAppUi()
    {

    delete iPositionRequestor;
    
    iLocationsDB.Close();
    
    iLocationArray->ResetAndDestroy();
    delete iLocationArray;

    if (iTimeLabel) delete iTimeLabel;

    if (iNaviContainer) 
        {
        iNaviContainer->Pop();
        }

    
    delete iHttpEngine;
    delete iTxtFormatDelete;
    delete iTxtDeleteLocation;
    delete iTxtModifyLocation;

    delete GLOBALS;


   }

// ------------------------------------------------------------------------------
// CWeatherAppUi::::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
//  This function is called by the EIKON framework just before it displays
//  a menu pane. Its default implementation is empty, and by overriding it,
//  the application can set the state of menu items dynamically according
//  to the state of application data.
// ------------------------------------------------------------------------------
//


void CWeatherAppUi::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane)
    {

    if (aResourceId == R_MAIN_WEATHER_MENU)
        {
        if (iCurrentLocation == NULL)
            {
            aMenuPane->DeleteMenuItem(ECmdRawData);
            aMenuPane->DeleteMenuItem(ECmdLocationInfo);
            aMenuPane->DeleteMenuItem(ECmdReload);            
            }
        else if (iCurrentLocation->GetReportString().Length() == 0)
            {
            aMenuPane->DeleteMenuItem(ECmdRawData);
            }
        }
    else if (aResourceId == R_CURRENT_LOCATION_SUBMENU)
        {
        TInt size = iLocationArray->Count();
        if (size == 0) return;
        TKeyArrayPtr locNameKey(_FOFF(CLocation,iDescription.iPlaceName), ECmpFolded);
        iLocationArray->Sort(locNameKey); 
        CEikMenuPaneItem::SData item;
        
        item.iCascadeId = 0;
        item.iFlags = 0;
        
        TInt i;
        for (i = 0; i < size ; i++)
            {
            CLocation* location = iLocationArray->At(i);
            item.iText = location->Name();
            item.iCommandId = 1000 + i;
            aMenuPane->AddMenuItemL(item);
            }
        }
    }


void CWeatherAppUi::ChooseFromCountryListCmdL()
    {

    CDesCArrayFlat* countries = iLocationsDB.GetAllCountriesL();
    TInt selected;
    TBuf<80> msg;
    iCoeEnv->ReadResource(msg, R_TEXT_CHOOSE_COUNTRY);
    if (ShowPopupListL(countries, msg, selected))
        {        
        TBuf<80> selectedCountry = (*countries)[selected];
        delete countries;
        
        TBuf<80> selectedState;
        CDesCArrayFlat* states = iLocationsDB.GetStatesInCountryL(selectedCountry);
        if (states->Count() > 0) 
            {
            iCoeEnv->ReadResource(msg, R_TEXT_CHOOSE_STATE);
            if (ShowPopupListL(states, msg, selected))
                {
                selectedState = (*states)[selected];
                RDebug::Print(_L("chose %S"), &selectedState);
                }
            }
        delete states;
        RPointerArray<TLocationDescription>* locDescs = 
            iLocationsDB.GetLocationsInCountryL(selectedCountry, selectedState);
        CleanupStack::PushL(locDescs);
        CDesCArrayFlat* locationNames = new (ELeave) CDesCArrayFlat(locDescs->Count());
        CleanupStack::PushL(locationNames);
        TInt i;
        for (i = 0; i < locDescs->Count() ; i++)
            {
            TLocationDescription* locDesc = (*locDescs)[i];
            locationNames->AppendL(locDesc->iPlaceName);
            }
        iCoeEnv->ReadResource(msg, R_TEXT_CHOOSE_STATION);
        if (ShowPopupListL(locationNames, msg, selected))
            {
            TPtrC selectedIcao = ((*locDescs)[selected])->iIcao;
            RDebug::Print(_L("chose %S"), &selectedIcao);
            CLocation* location = new(ELeave) CLocation( (*(*locDescs)[selected]) );
            CleanupStack::PushL(location);
            iLocationArray->AppendL(location);
            iCurrentLocation = location;
            DisplayLocationL();
            CleanupStack::Pop();
            }
        CleanupStack::Pop(locationNames);
        CleanupStack::Pop(locDescs);
        for (i = 0; i < locDescs->Count() ; i++)
            {
            delete (*locDescs)[i];
            }
        locDescs->Close();    
        delete locDescs;

        delete locationNames;
        }
    else 
        {
        RDebug::Print(_L("chose nothing"));
        delete countries;
        }
    
    }

_LIT(KRequestor,"Weather");
void CWeatherAppUi::ChooseByGpsL()
    {

    RDebug::Print(_L("!! creating wait dialog!!\n"));
    iWaitDialog = new( ELeave )CAknWaitDialog( REINTERPRET_CAST( CEikDialog**, & iWaitDialog ) ); 
    iWaitDialog->SetCallback(iPositionRequestor);
    iWaitDialog->ExecuteLD( R_GETTING_GPS_POS_NOTE );
    
    iPositionRequestor->StartRequest();
    
    }


void CWeatherAppUi::EditLocationsCmdL()
    {
    CAknListQueryDialog* dlg;
    TInt index = 0;
    dlg = new( ELeave ) CAknListQueryDialog( &index );

    if(dlg->ExecuteLD(R_EDIT_LOCATIONS_DIALOG))
        {
        RDebug::Print(_L("%d selected from edit locations"), index);
        
        switch(index)
            {
            case 0 :    //new
                {
#ifdef HAVE_LOCATION_CAPABILITY 
                dlg = new( ELeave ) CAknListQueryDialog( &index );
                if(dlg->ExecuteLD(R_ADD_LOCATION_METHOD_DIALOG))
                    {
                    switch(index)
                        {
                        case 0:
                            {
                            RDebug::Print(_L("want to use GPS"));
                            ChooseByGpsL();
                            break;
                            }
                        case 1:
                            {
                            RDebug::Print(_L("want to choose from list"));
                            ChooseFromCountryListCmdL();
                            break;
                            }
                            
                        }
                    } //dlg
#else
                ChooseFromCountryListCmdL();
#endif                                
                break;
                }
            case 1:     //delete
                {
                TInt selected;
                if (ShowLocationsPopupListL(*iTxtDeleteLocation, selected))
                    {
                    CLocation* location = iLocationArray->At(selected);
                    TBool deletingCurrent = (location == iCurrentLocation);
                    CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::ENoTone );
                    CleanupStack::PushL(dlg);
                    dlg->PrepareLC(R_DELETE_LOCATION_CONFIRMATION_QUERY_DIALOG);
                    TBuf<80> deleteString;
                    deleteString.Format(*iTxtFormatDelete, &(location->Name()));
                    dlg->SetPromptL(deleteString); 
                    if (dlg->RunLD())
                        {
                        //RDebug::Print(_L("delete %d"), selected);
                        iLocationArray->Delete(selected);
                        delete location;
                        }
                    CleanupStack::Pop();
                    if (deletingCurrent)
                        {
                        iCurrentLocation = NULL;
                        DisplayLocationL();
                        }
                    }
                break;
                }
            }
        }
    }


TBool CWeatherAppUi::ShowPopupListL(CDesCArrayFlat* aItems, const TDesC& aTitle, TInt& aSelectedIndex)
    {

    TInt size = aItems->Count();

    __ASSERT_ALWAYS((size > 0),  User::Panic(KEmptyLocations, 0) ); //TODO wrong error

    //only 1 element in list, so no point displaying list
    if (size == 1) 
        {
        aSelectedIndex = 0;
        return ETrue;
        }

    CAknSinglePopupMenuStyleListBox* list = new(ELeave) CAknSinglePopupMenuStyleListBox;
    CleanupStack::PushL(list);

    CAknPopupList* popupList = CAknPopupList::NewL(
                                        list, R_AVKON_SOFTKEYS_OK_BACK,
                                        AknPopupLayouts::EMenuWindow);
    CleanupStack::PushL(popupList);

    popupList->SetTitleL(aTitle);

    // initialize listbox.
    list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
    list->CreateScrollBarFrameL(ETrue);
    list->ScrollBarFrame()->SetScrollBarVisibilityL(
                                CEikScrollBarFrame::EOff,
                                CEikScrollBarFrame::EAuto);
    list->ItemDrawer()->FormattedCellData()->EnableMarqueeL( ETrue );
    
    CTextListBoxModel* model = list->Model();

    
    model->SetItemTextArray(aItems);
    model->SetOwnershipType(ELbmDoesNotOwnItemArray);

    TInt popupOk = popupList->ExecuteLD();
    CleanupStack::Pop();    //popuplist

    aSelectedIndex = list->CurrentItemIndex();

    CleanupStack::PopAndDestroy();  // list

    return popupOk;

    }

TBool CWeatherAppUi::ShowLocationsPopupListL(const TDesC& aTitle, TInt& aSelectedIndex)
    {

    TInt size = iLocationArray->Count();
    
    CDesCArrayFlat* locationNames = new (ELeave) CDesCArrayFlat(size);
    CleanupStack::PushL(locationNames);

    TInt i;
    for (i = 0; i < size ; i++)
        {
        CLocation* location = iLocationArray->At(i);
        locationNames->AppendL(location->Name());
        }
    
    TBool result = ShowPopupListL(locationNames, aTitle, aSelectedIndex);
    CleanupStack::Pop();
    delete locationNames;
    return result;

    }


void CWeatherAppUi::DisplayLocationL()
    {
    
    iWeatherView->SetLocation(iCurrentLocation);
    iLocationView->SetLocation(iCurrentLocation);
    
/*
    {
    CAknGlobalNote* globalNote = CAknGlobalNote::NewLC();
    globalNote->ShowNoteL(EAknGlobalInformationNote, _L("CWeatherAppUi::DisplayLocationL 1"));
    CleanupStack::PopAndDestroy();
    }
    */
    
    CEikStatusPane* sp = iEikonEnv->AppUiFactory()->StatusPane();
    
    CAknTitlePane* title = (CAknTitlePane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidTitle));
    if (iCurrentLocation)
        {
        title->SetTextL(iCurrentLocation->Name());
        }
    else
        {
        title->SetTextL(_L(""));
        }
    CAknContextPane* cp=(CAknContextPane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidContext));
 
    cp->SetPictureToDefaultL(); // Set application icon.
    sp->DrawNow(); 
    
    iNaviContainer = (CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi)); 
    iNaviContainer->Pop();
    if (iTimeLabel) delete iTimeLabel;
    iTimeLabel = NULL;

    
    if (!iCurrentLocation) 
        {
        return;
        }
    
    CWeatherInfo* report = iCurrentLocation->Report();

    if (!report)
        {
        return;
        }
    
    if (!iEikonEnv->ScreenDevice()->CurrentScreenMode()) //TODO this might not be the best way of checking
        {
        TInt temperature;
        
        if (report->GetTemperature(temperature))
            {
            CFbsBitmap* bitmap;
            CFbsBitmap* maskBitmap;
            CThermometer::DrawTemperatureL(temperature, bitmap, maskBitmap);
            cp->SetPicture(bitmap, maskBitmap);
            }

        }
    
    //_LIT(KDateFormat,"%*E%*D%X%*N%*Y %1 %2 '%3 %H:%T");
    _LIT(KDateFormat,"%*E%*D%X%*N %1 %2 %-B%:0%J%:1%T%:3%+B");


    TTime reportTime;
    if (report->GetTime(reportTime))
        {
        //convert to local time
        TLocale locale;
        TTimeIntervalSeconds universalTimeOffset(locale.UniversalTimeOffset());
        reportTime += universalTimeOffset;

        TBuf<40> dateString;
        reportTime.FormatL(dateString, KDateFormat);
        iTimeLabel = iNaviContainer->CreateNavigationLabelL(dateString);
        iNaviContainer->PushL(*iTimeLabel); 

        }

    
    //iWeatherView->DrawNow();

    }

_LIT(KWeatherServerName, "weather.noaa.gov");
_LIT8(KWeatherPathFormat, "/pub/data/observations/metar/stations/%S.TXT");
_LIT8(KWeatherPathProxyFormat, "http://weather.noaa.gov/pub/data/observations/metar/stations/%S.TXT");

void CWeatherAppUi::DownloadLocationL()
    {
    RDebug::Print(_L("loading %S!\n"), &iCurrentLocation->IcaoCode());

    TBuf8<KMaxPathLength> path;
    TBuf8<4> icao;

    CnvUtfConverter::ConvertFromUnicodeToUtf8(icao, iCurrentLocation->IcaoCode());

    RDebug::Print(_L("!! creating wait dialog!!\n"));
    iWaitDialog = new( ELeave )CAknWaitDialog( REINTERPRET_CAST( CEikDialog**, & iWaitDialog ) ); 
    iWaitDialog->SetCallback(iHttpEngine);
    iWaitDialog->ExecuteLD( R_LOADING_REPORT_NOTE );

    if (GLOBALS->iSettings.iUseProxy)
        {
        TBuf<20> buf;
        GLOBALS->iSettings.iProxyIpAddress.Output(buf);
        
        path.AppendFormat(KWeatherPathProxyFormat, &icao);
        iHttpEngine->StartHttpGetL(buf, path, GLOBALS->iSettings.iProxyPort);
        }
    else
        {
        path.AppendFormat(KWeatherPathFormat, &icao);
        iHttpEngine->StartHttpGetL(KWeatherServerName, path, 80);
        }
    }

void CWeatherAppUi::CurrentLocationCmdL(TInt aIndex)
    {
    
    RDebug::Print(_L("current location changed to %d"), aIndex);

    iLastLocationIndex = aIndex;

    CLocation* previousLocation = iCurrentLocation;
    iCurrentLocation = iLocationArray->At(aIndex);

    //if there is no report loaded, ask if they want to load one
    if (iCurrentLocation && iCurrentLocation->GetReportString().Length() == 0)
        {
        CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::ENoTone );
        CleanupStack::PushL(dlg);
        dlg->PrepareLC(R_RELOAD_LOCATION_QUERY_DIALOG);
        if (dlg->RunLD())
            {
            DownloadLocationL();
            }
        CleanupStack::Pop();
        }
    else if (iCurrentLocation != previousLocation) 
        {
        DisplayLocationL();
        }
    

    }

void CWeatherAppUi::SocketStatusNotify(const TDesC& aMessage, TBool aTerminal)
    {
    if (aTerminal) 
        {
        if (iWaitDialog)
            {
            iWaitDialog->ProcessFinishedL(); 
            iWaitDialog = NULL;
            }
        CAknInformationNote* note = new (ELeave)CAknInformationNote();    
        note->ExecuteLD(aMessage);
        }
    else 
        {
        if (iWaitDialog)
            {
            iWaitDialog->SetTextL(aMessage);
            }
        }
    }


void CWeatherAppUi::LbsPositionInfoUpdatedL(TPositionInfo& aPosInfo)
    {

    if (iWaitDialog)
        {
        iWaitDialog->ProcessFinishedL(); 
        iWaitDialog = NULL;
        }
    
    TPosition position;
    aPosInfo.GetPosition(position);
    RPointerArray<TLocationDescription>* locDescs = 
        iLocationsDB.FindNearbyLocationsL(position.Latitude(), position.Longitude());
    TInt i;
    CDesCArrayFlat* locationNames = new (ELeave) CDesCArrayFlat(locDescs->Count());
    CleanupStack::PushL(locationNames);
    TInt selected;
    for (i = 0; i < locDescs->Count() ; i++)
        {
        TLocationDescription* locDesc = (*locDescs)[i];
        locationNames->AppendL(locDesc->iPlaceName);
        }
    TBuf<80> msg;
    iCoeEnv->ReadResource(msg, R_TEXT_CHOOSE_STATION);    
    if (ShowPopupListL(locationNames, msg, selected))
        {
        TPtrC selectedIcao = ((*locDescs)[selected])->iIcao;
        RDebug::Print(_L("chose %S"), &selectedIcao);
        CLocation* location = new(ELeave) CLocation((*(*locDescs)[selected]));
        CleanupStack::PushL(location);
        iLocationArray->AppendL(location);
        iCurrentLocation = location;
        DisplayLocationL();
        CleanupStack::Pop();    
        }
    CleanupStack::Pop();
    for (i = 0; i < locDescs->Count() ; i++)
        {
        delete (*locDescs)[i];
        }
    locDescs->Close();
    delete locDescs;

    delete locationNames;
    
    
    }
void CWeatherAppUi::LbsShowErrorL(const TDesC& aErrorString)
    {
    if (iWaitDialog)
        {
        iWaitDialog->ProcessFinishedL(); 
        iWaitDialog = NULL;
        }
    CAknInformationNote* note = new (ELeave)CAknInformationNote();    
    note->ExecuteLD(aErrorString);
    }

void CWeatherAppUi::LbsShowMessageL(const TDesC& aMessage)
    {
    if (iWaitDialog)
        {
        iWaitDialog->ProcessFinishedL(); 
        iWaitDialog = NULL;
        }
    CAknInformationNote* note = new (ELeave)CAknInformationNote();    
    note->ExecuteLD(aMessage);
    }


void CWeatherAppUi::LbsCancelling(void)
    {
    if (iWaitDialog)
        {
        TBuf<40> msg;
        iCoeEnv->ReadResource(msg, R_TEXT_CANCELLING);
        iWaitDialog->SetTextL(msg); 
        }
    }

void CWeatherAppUi::LbsCancelled(void)
    {
    if (iWaitDialog)
        {
        iWaitDialog->ProcessFinishedL(); 
        iWaitDialog = NULL;
        }
    }

void CWeatherAppUi::SocketStatusNotify(TInt aMsgResourceId, TBool aTerminal)
    {
    TBuf<100> msgBuf; 
    iCoeEnv->ReadResource(msgBuf, aMsgResourceId);
    SocketStatusNotify(msgBuf, aTerminal); 
    }

void CWeatherAppUi::HttpLoadedNotify(TPtrC aBody, TInt aResponseCode)
    {
  
    RDebug::Print(_L("HttpLoadedNotify\n"));
    switch(aResponseCode)
        {
        case 200 : 
            {
            TInt start = 0;
            while ((aBody[start] == '\n') || (aBody[start] == '\r')) start++;
            TPtrC trimmed = aBody.Right(aBody.Length() - start);
            HBufC* reportString = trimmed.AllocLC();            
            iCurrentLocation->ParseReportL(reportString);       //ownership of string transferred
            //must check dialog is still around as it may have been cancelled
            if (iWaitDialog)
                {
                iWaitDialog->ProcessFinishedL(); 
                iWaitDialog = NULL;
                }
            SaveLocationsL();
            CleanupStack::Pop();
            DisplayLocationL();     
            break;
            }
        case 404: SocketStatusNotify(R_TEXT_FILE_NOT_FOUND, ETrue);   break;
        
        default:  
            {
            TBuf<40> msgBuf;
            TBuf<40> msgFmt;
            iCoeEnv->ReadResource(msgFmt, R_TEXT_UNEXPECTED_RESPONSE_CODE);
            msgBuf.Format(msgFmt, aResponseCode);
            SocketStatusNotify(msgBuf, ETrue); 


            }
        }
    }

void CWeatherAppUi::HandleResourceChangeL( TInt aType )
    {
    CAknAppUi::HandleResourceChangeL( aType );
       
    if ( aType==KEikDynamicLayoutVariantSwitch )
        {
        iWeatherView->HandleSizeChange(aType);
        iLocationView->HandleSizeChange(aType);
        }           
    }


TKeyResponse CWeatherAppUi::HandleKeyEventL(
    const TKeyEvent& /*aKeyEvent*/,TEventCode /*aType*/)
    {
    return EKeyWasNotConsumed;
    }


void CWeatherAppUi::HandleCommandL(TInt aCommand)
    {
    switch ( aCommand )
        {
        case EAknSoftkeyExit:
        case EEikCmdExit:
            {
            SaveLocationsL();
            SaveUserSettingsL();
            Exit();
            break;
            }

        case ECmdUserSettings:
            {
            CUserSettingsDialog* dialog = new(ELeave) CUserSettingsDialog(this);
            dialog->PrepareLC(R_USER_SETTINGS_DIALOG);
            dialog->ConstructL();
            dialog->RunLD();
            SaveUserSettingsL();
            break;
            }

        case ECmdEditLocation:
            {
            EditLocationsCmdL();
            SaveLocationsL();
            break;
            }

        case ECmdLocationInfo:
            {
            if (iCurrentLocation)
                {
                ActivateLocalViewL( TUid::Uid( ELocationViewId ) );
                }
            break;            
            }

        case ECmdRawData:
            {
            if (iCurrentLocation && iCurrentLocation->GetReportString().Length() > 0)
                {
                HBufC* title = iEikonEnv->AllocReadResourceLC(R_TEXT_RAW_METAR_REPORT); 
                CAknMessageQueryDialog* dlg = CAknMessageQueryDialog::NewL(iCurrentLocation->GetReportString());
                dlg->PrepareLC(R_RAW_METAR_DIALOG);
                //_LIT( KDlgName, "Landmark Info" );
                dlg->QueryHeading()->SetTextL(title->Des()); 
                dlg->RunLD();
                CleanupStack::PopAndDestroy(title);
                }
            break;            
            }
            
        case ECmdAbout:
            {
            CAboutDialog* dialog = new(ELeave) CAboutDialog();
            //dialog->SetMopParent(this);
            dialog->ExecuteLD(R_ABOUT_BOX);
            break;
            }

        case ECmdReload:
            {
            DownloadLocationL();
            break;
            }
            
        default:
            {
            if ((aCommand >= 1000) && (aCommand < 1000+iLocationArray->Count()))
                {
                __ASSERT_ALWAYS((iLocationArray->Count() > 0),  User::Panic(KEmptyLocations, 1) );
                CurrentLocationCmdL(aCommand-1000);
                }
            }
            break;      
        }
    }

// End of File  
