/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

// INCLUDE FILES
#include <eiklabel.h>
#include <gulfont.h>
#include <aknscontrolcontext.h> 
#include <aknsbasicbackgroundcontrolcontext.h> 
#include <aknnotewrappers.h> 
#include <aknglobalnote.h> 

#include "weathercontainer.h"
#include <weather.rsg>
#include "weather.hrh"
#include "weatherappui.h"
#include "location.h"

CWeatherContainer::CWeatherContainer()
    {


    }

// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CWeatherContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CWeatherContainer::ConstructL(const TRect& aRect)
    {

    iBgContext = CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain,aRect,ETrue);
    
    CreateWindowL();

    SetRect(aRect);

    iWeatherDisplay = new (ELeave) CWeatherDisplay();

    iWeatherDisplay->SetContainerWindowL(*this);
    iWeatherDisplay->ConstructL(aRect);


    iSBFrame = new (ELeave) CEikScrollBarFrame(this, this, ETrue);
    iSBFrame->CreateDoubleSpanScrollBarsL(ETrue, EFalse);
    iSBFrame->SetTypeOfVScrollBar(CEikScrollBarFrame::EDoubleSpan);   
    iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EOn);

    iYTop = 0;

    ActivateL();

    }

void CWeatherContainer::HandleScrollEventL(CEikScrollBar* /*aScrollBar*/, TEikScrollEvent /*aEventType*/)
    {
    }


void CWeatherContainer::SetLocation(CLocation* aLocation)
    {

    iWeatherDisplay->SetLocation(aLocation);

    iYTop = 0;
    iWeatherDisplay->SetPosition(TPoint(0, iYTop));
    UpdateScrollbars();
    
    DrawNow();
    iWeatherDisplay->DrawNow();

    }


void CWeatherContainer::UpdateScrollbars()
    {

    TRect rect = iWeatherDisplay->Rect();
    TEikScrollBarModel hSbarModel;
    TEikScrollBarModel vSbarModel;
    TEikScrollBarFrameLayout layout;

    
    if (rect.Height() > Rect().Height()) 
    {

        RDebug::Print(_L("*********** should be on!"));
        TRect containerRect = Rect();
        
        vSbarModel.iThumbSpan = containerRect.Height();
        vSbarModel.iScrollSpan = rect.Height();
        vSbarModel.iThumbPosition = -iYTop;

        layout.iTilingMode=TEikScrollBarFrameLayout::EInclusiveRectConstant;
        iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EOn);
        
        
        iSBFrame->TileL(&hSbarModel, &vSbarModel, containerRect/*rect*/, containerRect/*rect*/, layout);
        iSBFrame->SetVFocusPosToThumbPos(vSbarModel.iThumbPosition);

    }
    else {
        RDebug::Print(_L("*********** should be off!"));
        iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EOff);

        iSBFrame->TileL(&hSbarModel, &vSbarModel, rect, rect, layout);
    }

    RDebug::Print(_L("iYTop=%d  containerheight=%d  viewheight=%d"), iYTop, Rect().Height(), rect.Height());
    iSBFrame->DrawScrollBarsNow();
    
    ActivateL();

    }


// Destructor
CWeatherContainer::~CWeatherContainer()
    {
    delete iBgContext;
    delete iSBFrame;
    delete iWeatherDisplay;

    }


TTypeUid::Ptr CWeatherContainer::MopSupplyObject(TTypeUid aId)
    {
    if (iBgContext )
        {
        return MAknsControlContext::SupplyMopObject( aId, iBgContext );
        }
    return CCoeControl::MopSupplyObject(aId);
    }

// ---------------------------------------------------------
// CWeatherContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CWeatherContainer::SizeChanged()
    {
    if(iBgContext)
        {
        iBgContext->SetRect(Rect());
        if (&Window())
            {
            iBgContext->SetParentPos( PositionRelativeToScreen() );
            }
        }   
    if (iWeatherDisplay) 
        {
        iWeatherDisplay->SetRect(Rect());
        iWeatherDisplay->ReLayout();
        }
    }

void CWeatherContainer::HandleResourceChange(TInt aType)
    {
    RDebug::Print(_L("!HandleResourceChange"));
    CCoeControl::HandleResourceChange(aType);
    // ADDED FOR SCALABLE UI SUPPORT
    // *****************************
    if (aType==KEikDynamicLayoutVariantSwitch)
        {
        TRect rect;
        AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane, rect);
        SetRect(rect);
        UpdateScrollbars();
        }
    }

// ---------------------------------------------------------
// CWeatherContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CWeatherContainer::CountComponentControls() const
    {
    return 1;
    }

// ---------------------------------------------------------
// CWeatherContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CWeatherContainer::ComponentControl(TInt /*aIndex*/) const
    {

    return iWeatherDisplay;

    }

// ---------------------------------------------------------
// CWeatherContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CWeatherContainer::Draw(const TRect& aRect) const
    {

    CWindowGc& gc = SystemGc();

    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    MAknsControlContext* cc = AknsDrawUtils::ControlContext(this);
    AknsDrawUtils::Background(skin, cc, this, gc, aRect);
    
    }

// ---------------------------------------------------------
// CWeatherContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CWeatherContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
    {
    }


TKeyResponse CWeatherContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
    {

    if (aType != EEventKey) return EKeyWasNotConsumed;

    TInt containerHeight = Rect().Height();
    TInt viewHeight = iWeatherDisplay->Rect().Height();

    if (aKeyEvent.iCode==EKeyDownArrow && 
            (viewHeight > containerHeight) &&  (iYTop > (containerHeight - viewHeight)) )
        {
        RDebug::Print(_L("down"));
        iYTop -= 22;
        iWeatherDisplay->SetPosition(TPoint(0, iYTop));
        UpdateScrollbars();
        DrawNow();
        //iWeatherDisplay->DrawNow();
        return EKeyWasConsumed;
        }
    if (aKeyEvent.iCode==EKeyUpArrow &&
            (iYTop < 0 ))
        {
        RDebug::Print(_L("up"));
        iYTop += 22;
        iWeatherDisplay->SetPosition(TPoint(0, iYTop));
        UpdateScrollbars();
        DrawNow();
        //iWeatherDisplay->DrawNow();
        return EKeyWasConsumed;
        }

    return EKeyWasNotConsumed;
    }


// End of File  
