
/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


// INCLUDE FILES
#include <weather.rsg>
#include <aknutils.h> 
#include "weather.hrh"
#include <eiklabel.h>
#include <gulfont.h>
#include <e32math.h>
#include <aknsutils.h> 
#include <aknappui.h> 
#include <aknglobalnote.h> 

#include "location.h"
#include "weatherdisplay.h"
#include "weatherappui.h"
#include "weatherutils.h"

CWeatherDisplay::CWeatherDisplay()
    {


    }

// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CWeatherView::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CWeatherDisplay::ConstructL(const TRect& aRect)
    {
    
    iControlArray = new (ELeave) CArrayPtrFlat<CCoeControl> (5); 

    iMaxRect = aRect;
    iMaxRect.iTl.iY = 0;
    iMaxRect.iBr.iY -= 80;
    SetRect(iMaxRect);


    ActivateL();

    iDenseFont = iEikonEnv->LegendFont();
    
    TResourceReader reader;
                                            
    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_PHENOMENA);
    iPhenomenonDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iPhenomenonDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_MODIFIERS);
    iModifierDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iModifierDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_INTENSITY);
    iIntensityDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iIntensityDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_CLOUDS);
    iCloudDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iCloudDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_COMPASS);
    iCompassDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iCompassDict
    CleanupStack::PopAndDestroy();    //reader 


    }


void CWeatherDisplay::AddLabelL(const TDesC& aString, TPoint& aPos, TInt aWidth)
    {
    CBitmapLabel* label = new (ELeave) CBitmapLabel;
    label->ConstructL(iDenseFont, aString, aWidth, 0);   
    label->SetContainerWindowL( *this );
    label->SetExtent( aPos, label->MinimumSize());
    aPos.iY += label->Size().iHeight + 5;
    iControlArray->AppendL(label);
    }

_LIT(KNoMatchPanic, "No match");
void CWeatherDisplay::AddConditionLabelL(TWeatherCondition& aCondition, TPoint& aPos, TInt aWidth)
    {

    iTempDesc1 = _L("");    //the tendency
    if (aCondition.iTrend == ETemporary)
        {
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_TEMPORARY);
        }
    else if (aCondition.iTrend == EBecoming)
        {
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_BECOMING);
        }
    
    
    HBufC* phenomString = iPhenomenonDict->GetStringForInt(aCondition.iPhenomenon);
    __ASSERT_ALWAYS(phenomString, User::Panic(KNoMatchPanic, 1));
    
    
    HBufC* intensityString = iIntensityDict->GetStringForInt(aCondition.iIntensity);
    __ASSERT_ALWAYS(intensityString, User::Panic(KNoMatchPanic, 2));
    
    
    HBufC* modifierString = iModifierDict->GetStringForInt(aCondition.iModifier);
    __ASSERT_ALWAYS(modifierString, User::Panic(KNoMatchPanic, 3));
    
    
    //phenonString contains the format - e.g. "%S%Swidespread dust"            
    RDebug::Print(*phenomString, &iTempDesc1, intensityString, modifierString);
    
    iTempDesc2.Format(*phenomString, &iTempDesc1, intensityString, modifierString);
    iTempDesc2.Capitalize();
    

    TInt bitmapId = -1; //no image

    //choose image
    switch(aCondition.iPhenomenon)
        {
        case EDz:
        case ERa:
            bitmapId = EMbmWeatherRain;
            break;

        case ESn:
        case ESg:
        case EIc:
            bitmapId = EMbmWeatherSnow;
            break;

        case EPe:
        case EGr:
        case EGs:
            bitmapId = EMbmWeatherHail;
            break;

        case EFu:
            bitmapId = EMbmWeatherSmoke;
            break;
            
        case EBr:
        case EFg:
        case EHz:
            bitmapId = EMbmWeatherFog;
            break;

        case EVa:
            bitmapId = EMbmWeatherVolcano;
            break;
            
        default:
            break;

        }
    
    CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
    CleanupStack::PushL(bitmapLabel);
    bitmapLabel->ConstructL(iDenseFont, iTempDesc2, aWidth, bitmapId);
    bitmapLabel->SetContainerWindowL( *this );
    bitmapLabel->SetExtent( aPos, bitmapLabel->MinimumSize() );
    aPos.iY += bitmapLabel->Size().iHeight + 5;
    iControlArray->AppendL(bitmapLabel);
    CleanupStack::Pop(); //bitmapLabel

    
    }


void CWeatherDisplay::AddCloudSummaryLabelL(CArrayFixFlat<TCloud>* aClouds, TPoint& aPos, TInt aWidth)
    {

    TInt size;
    if (!aClouds)
        {
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_NO_CLOUD_INFORMATION);
        return AddLabelL(iTempDesc1, aPos, aWidth);      
        }

    size = aClouds->Count();

    TCloudCoverage heaviest = EClr;
    TInt i;
    for (i = 0; i < size; i++)
        {
        TCloud cloud = aClouds->At(i);
        if (cloud.iCoverage > heaviest)
            heaviest = cloud.iCoverage;

        }
    

    TInt bitmapId;
    HBufC* cloudSummary;

    switch(heaviest)
        {
        
        case EFew:
        case ESct:
            {
            bitmapId = EMbmWeatherScatteredclouds;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_SOME_CLOUDS);
            break;
            }
        case EBkn:
            {
            bitmapId = EMbmWeatherMostlycloudy;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_MOSTLY_CLOUDY);
            break;
            }
        case EOvc:
            {
            bitmapId = EMbmWeatherOvercast;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_OVERCAST_CLOUDS);
            break;
            }
        default:    //clear skies
            {
            bitmapId = EMbmWeatherSun;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_NO_CLOUDS);
            break;
            }
        }

    CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
    CleanupStack::PushL(bitmapLabel);
    bitmapLabel->ConstructL(iDenseFont, *cloudSummary, aWidth, bitmapId);
    bitmapLabel->SetContainerWindowL( *this );
    bitmapLabel->SetExtent( aPos, bitmapLabel->MinimumSize() );
    aPos.iY += bitmapLabel->Size().iHeight + 5;
    iControlArray->AppendL(bitmapLabel);
    CleanupStack::Pop(); //bitmapLabel
    delete cloudSummary;
    
    }



void CWeatherDisplay::AddCloudLabelL(TCloud& cloud, TPoint& aPos, TInt aWidth)
    {
    
    HBufC* cloudString = iCloudDict->GetStringForInt(cloud.iCoverage);
    __ASSERT_ALWAYS(cloudString, User::Panic(KNoMatchPanic, 4));

    
    double heightFraction;   
    if (GLOBALS->iSettings.iDistanceUnits == EMiles)
        {
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_AT_MILES);
        heightFraction = FeetToMiles(cloud.iHeight);
        }
    else    //kilometres
        {
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_AT_KILOMETRES);
        heightFraction = ((float)FeetToMetres(cloud.iHeight))/1000;
        }
    
    iTempDesc2.Format(iTempDesc1, heightFraction);

    RDebug::Print(iTempDesc2);

    TBuf<80> description;
    description.Format(*cloudString, &iTempDesc2);
       
 
    RDebug::Print(*cloudString, &iTempDesc2);
    RDebug::Print(description);

    TInt bitmapId = -1;

    switch(cloud.iCoverage)
        {
        case EFew:
            bitmapId = EMbmWeatherFewclouds;
            break;
        case ESct:
            bitmapId = EMbmWeatherScatteredclouds;
            break;
        case EBkn:
        case EOvc:
            bitmapId = EMbmWeatherMostlycloudy;
            break;

        default:    //clear skies
            bitmapId = EMbmWeatherSun;
            break;
        }



    CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
    CleanupStack::PushL(bitmapLabel);
    
    bitmapLabel->ConstructL(iDenseFont, description, aWidth, bitmapId);
    bitmapLabel->SetContainerWindowL( *this );
    bitmapLabel->SetExtent( aPos, bitmapLabel->MinimumSize() );
    aPos.iY += bitmapLabel->Size().iHeight + 5;
    iControlArray->AppendL(bitmapLabel);
    CleanupStack::Pop(); //bitmapLabel   
    
    }


//_LIT(KDateFormat,"%E%D%X%N%Y %1 %2 %3 %H:%T");
_LIT(KDateFormat,"%*E%*D%X%*N%*Y %1 %2 '%3 %H:%T");

_LIT(KNoReportFormat, "No report for %S");

void CWeatherDisplay::AddWindAndVisibilitySection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth)
    {
    TInt windSpeed, gustSpeed;
    TCompassDirection direction;
    TBool directionVariable;
    if (aReport->GetWind(windSpeed, direction, gustSpeed, directionVariable))
        {
        //iTempDesc1 holds "Wind %S"
        //iTempDesc2 holds the final string
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_WIND);

        if (windSpeed == 0)
            {
            TBuf<10> calmBuf;
            iCoeEnv->ReadResource(calmBuf, R_TEXT_CALM);
            iTempDesc2.Format(iTempDesc1, &calmBuf);
            RDebug::Print(_L("Wind: Calm"));
            //AddLabelL( _L("Wind: Calm"), yPos);
            }
        else 
            {
            TBuf<20> buf;
            MakeSpeedString(buf, windSpeed);
            RDebug::Print(_L("windSpeed is %S"), &buf);
            iTempDesc2.Format(iTempDesc1, &buf);

            if (directionVariable)
                {
                iCoeEnv->ReadResource(buf, R_TEXT_DIRECTION_VARIES);
                RDebug::Print(_L("Direction varies"));
                iTempDesc2.Append(buf);
                }
            else
                {
                HBufC* compassString = iCompassDict->GetStringForInt(direction);
                __ASSERT_ALWAYS(compassString, User::Panic(KNoMatchPanic, 0));
                RDebug::Print(_L("%S"), compassString);
                iTempDesc2.Append(*compassString);
                }

            if (gustSpeed > 0) 
                {
                TBuf<10> gustsFmt;
                iCoeEnv->ReadResource(buf, R_TEXT_GUSTS_OF);
                MakeSpeedString(buf, gustSpeed);
                RDebug::Print(_L("gustSpeed is %S"), &buf);
                iTempDesc2.AppendFormat(gustsFmt, &buf);
                }
            }
            AddLabelL(iTempDesc2, aPos, aWidth);
        }
    else
        {
        iCoeEnv->ReadResource(iTempDesc2, R_TEXT_NO_WIND_INFORMATION);
        AddLabelL(iTempDesc2, aPos, aWidth);
        }  // wind


    TInt visibility;
    if (aReport->GetVisibility(visibility))
        {
        if (visibility == 9999)
            {
            iCoeEnv->ReadResource(iTempDesc2, R_TEXT_VISIBILITY_UNLIMITED);
            }
        else
            {
            double visibilityFraction;       
            if (GLOBALS->iSettings.iDistanceUnits == EMiles)
                {
                iCoeEnv->ReadResource(iTempDesc1, R_TEXT_VISIBILITY_MILES);
                visibilityFraction = MetresToMiles(visibility);
                }
            else    //kilometres
                {
                iCoeEnv->ReadResource(iTempDesc1, R_TEXT_VISIBILITY_KILOMETRES);
                visibilityFraction = ((float)visibility)/1000;
                }
            iTempDesc2.Format(iTempDesc1, visibilityFraction);
    
            }
            AddLabelL(iTempDesc2, aPos, aWidth);
        } //visibility
    }

void CWeatherDisplay::AddConditionSection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth)
    {
    CArrayFixFlat<TWeatherCondition>* conditions = aReport->GetConditions();
    if (conditions)
        {
        TInt size = conditions->Count();
        TInt i;
        for (i = 0; i < size; i++)
            {
            TWeatherCondition condition = conditions->At(i);
            AddConditionLabelL(condition, aPos, aWidth);
            }
        }
    }


void CWeatherDisplay::AddCloudSection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth)
    {
    CArrayFixFlat<TCloud>* clouds = aReport->GetClouds();

    if (clouds)
        {
        if (GLOBALS->iSettings.iDetailedCloudReports)
            {
            TInt size = clouds->Count();
            TInt i;
            for (i = 0; i < size; i++)
                {
                TCloud cloud = clouds->At(i);
                AddCloudLabelL(cloud, aPos, aWidth);
                }
            }
        else
            {
            AddCloudSummaryLabelL(clouds, aPos, aWidth);
            }

        }
    }

void CWeatherDisplay::AddHumidityDewPointAndPressureSection(CWeatherInfo* aReport, TPoint& aPos, TInt aWidth)
    {
    
    TInt millibars;
    if (aReport->GetPressure(millibars))
        {
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_PRESSURE_MB);
        RDebug::Print(iTempDesc1, millibars);
        iTempDesc2.Format(iTempDesc1, millibars);
        AddLabelL(iTempDesc2, aPos, aWidth);
        }
    
    TInt temperature;
    if (aReport->GetTemperature(temperature))
        {
        TInt dewPoint;
        if (aReport->GetDewPoint(dewPoint))
            {
            TBuf<10> tempBuf;
            MakeTempString(tempBuf, dewPoint);
        
            TReal relativeHumidity;            
            Math::Round(relativeHumidity, (RelativeHumidity((double)temperature, (double)dewPoint)*100), 0);
            iCoeEnv->ReadResource(iTempDesc1, R_TEXT_DEW_POINT_AND_HUMIDITY);
            iTempDesc2.Format(iTempDesc1, &tempBuf, (TInt)relativeHumidity);
            AddLabelL(iTempDesc2, aPos, aWidth);
            }
        }
    }


void CWeatherDisplay::SetLocation(CLocation* aLocation)
    {

    iLocation = aLocation;
    //clear out any previous controls
    iControlArray->ResetAndDestroy();
    iDrawSplitters = FALSE;

    TPoint pos(5, 5);
    TInt width = Size().iWidth - 10;
    
    if (!aLocation)
        {
        RDebug::Print(_L("no location"));
        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_NO_LOCATION_LOADED);

        AddLabelL(iTempDesc1, pos, width);
        
        return;

        }

    CWeatherInfo* report = aLocation->Report();


    if (!report)
        {
        RDebug::Print(_L("no report for %S"), &(aLocation->Name()));

        iCoeEnv->ReadResource(iTempDesc1, R_TEXT_NO_REPORT_FOR);
        iTempDesc2.Format(iTempDesc1, &(aLocation->Name()));
        AddLabelL(iTempDesc2, pos, width);
        return;
        }

    RDebug::Print(_L("report for %S"), &(aLocation->Name()));
    
    
    iDrawSplitters = TRUE;
    
    //neither of these methods of determining the screen orientation actually work reliably (typical for symbian...)
    //CAknAppUiBase::TAppUiOrientation  orientation = ((CAknAppUi*)CEikonEnv::Static()->AppUi())->Orientation();
    //if (orientation == CAknAppUi::EAppUiOrientationLandscape) 
    //if (iEikonEnv->ScreenDevice()->CurrentScreenMode())
    //so i am doing it the brain dead way...
    const TSize screenSize = iEikonEnv->ScreenDevice()->SizeInPixels();
    if (screenSize.iWidth > screenSize.iHeight)
        {
        //landscape mode
        
        width /=2;
        iXPosMidSplit = width;
        
        //LHS
        TPoint lhs = pos;
        TInt temperature;
        if (report->GetTemperature(temperature))
            {
            CFbsBitmap* thermBitmap;
            CFbsBitmap* thermMask;
            CThermometer::DrawTemperatureL(temperature, thermBitmap, thermMask);

            TSize bitmapSize = thermBitmap->SizeInPixels();
            bitmapSize = thermMask->SizeInPixels();
            
            CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
            CleanupStack::PushL(bitmapLabel);
            
            bitmapLabel->ConstructL(thermBitmap, thermMask);
            bitmapLabel->SetContainerWindowL( *this );
            bitmapLabel->SetExtent( lhs, bitmapLabel->MinimumSize());
            lhs.iY += bitmapLabel->Size().iHeight + 5;
            iControlArray->AppendL(bitmapLabel);
            CleanupStack::Pop(); //bitmapLabel   
            }
        else 
            {
            iCoeEnv->ReadResource(iTempDesc1, R_TEXT_NO_TEMPERATURE_INFO);
            AddLabelL(iTempDesc1, lhs, width);
            }
        
        //RHS
        pos.iX += width;
        AddWindAndVisibilitySection(report, pos, width);
        pos.iY = Max(lhs.iY, pos.iY);
        iTopBoxYBottom = pos.iY;
        pos.iY += 5;
        
        //LHS
        lhs = pos;
        lhs.iX -= width;
        AddConditionSection(report, lhs, width);
        //RHS
        AddHumidityDewPointAndPressureSection(report, pos, width);   
        pos.iY = Max(lhs.iY, pos.iY);
        iMidBoxYBottom = pos.iY;
        pos.iY += 5;
        
        //clouds section takes up the whole width of the screen
        pos.iX -= width;
        width *= 2;
        AddCloudSection(report, pos, width);
        
        } 
    else 
        {
        
        //portrait mode

        iXPosMidSplit = 0; //no vertical split
        
        AddWindAndVisibilitySection(report, pos, width);
        iTopBoxYBottom = pos.iY;
        pos.iY += 5;
        
        AddConditionSection(report, pos, width);
        pos.iY += 5;
        AddHumidityDewPointAndPressureSection(report, pos, width);   
        iMidBoxYBottom = pos.iY;        
        pos.iY += 5;
        
        AddCloudSection(report, pos, width);

        }
        
    
/* NOT USED

    TTime reportTime;
    if (report->GetTime(reportTime))
        {
        TBuf<40> dateString;
        reportTime.FormatL(dateString, KDateFormat);
        
        RDebug::Print(_L("date=%S"), &dateString);        
        }
*/
    
    pos.iY += 10;

    TRect rect = Rect();
    rect.iTl.iY = 0;
    rect.SetHeight(pos.iY);
    SetRect(rect);
    RDebug::Print(_L("top left x=%d y=%d  height=%d"), rect.iTl.iX, rect.iTl.iY, rect.Height());
    
    }


// Destructor
CWeatherDisplay::~CWeatherDisplay()
    {
    iControlArray->ResetAndDestroy();
    delete iControlArray;

    delete iPhenomenonDict;
    delete iModifierDict;
    delete iIntensityDict;
    delete iCloudDict;
    delete iCompassDict;

    }


void CWeatherDisplay::ReLayout()
    {

    if (iLocation) SetLocation(iLocation);
    //DrawDeferred();
    }


// ---------------------------------------------------------
// CWeatherView::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CWeatherDisplay::SizeChanged()
    {

    }

// ---------------------------------------------------------
// CWeatherView::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CWeatherDisplay::CountComponentControls() const
    {
    return iControlArray->Count();
    }

// ---------------------------------------------------------
// CWeatherView::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CWeatherDisplay::ComponentControl(TInt aIndex) const
    {
    return iControlArray->At(aIndex);

    }

// ---------------------------------------------------------
// CWeatherView::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CWeatherDisplay::Draw(const TRect& aRect) const
    {

    
    CWindowGc& gc = SystemGc();  
    gc.SetClippingRect(aRect);
    gc.SetPenStyle(CGraphicsContext::EDottedPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
    
    TRgb colour;
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    AknsUtils::GetCachedColor(skin, colour, KAknsIIDQsnTextColors, EAknsCIQsnTextColorsCG6);
    
    TInt yOffset = Position().iY;
    if (iDrawSplitters)
        {
        gc.SetPenColor(colour);    
        gc.DrawLine(TPoint(0,iTopBoxYBottom+yOffset), TPoint(Size().iWidth, iTopBoxYBottom+yOffset));
        gc.DrawLine(TPoint(0,iMidBoxYBottom+yOffset), TPoint(Size().iWidth, iMidBoxYBottom+yOffset));
        if (iXPosMidSplit > 0)
            {
            gc.DrawLine(TPoint(iXPosMidSplit, yOffset), TPoint(iXPosMidSplit, iMidBoxYBottom+yOffset));   
            }        
        }
    
    }


// End of File  
