
/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include "WeatherInfo.h"

CWeatherInfo* CWeatherInfo::NewLC()
    {
    CWeatherInfo* self = new (ELeave) CWeatherInfo();
    CleanupStack::PushL(self);

    return self;
    }


CWeatherInfo::CWeatherInfo()
    {

    iHasWind = EFalse;
    iDirectionVariable = EFalse;
    iHasTemperature = EFalse;
    iHasDewPoint = EFalse;
    iHasVisibility = EFalse;

    }

CWeatherInfo::~CWeatherInfo()
    {
    if (iConditions)
        delete iConditions;
    if (iClouds)
        delete iClouds;
    }


void CWeatherInfo::AddConditionL(TPhenomenonTrend aTrend,
                                 TIntensity aIntensity, 
                                 TPhenomenonModifier aModifier,
                                 TPhenomenon aPhenomenon)
    {
    TWeatherCondition condition;
    
    condition.iIntensity = aIntensity;
    condition.iModifier = aModifier;
    condition.iPhenomenon = aPhenomenon;
    condition.iTrend = aTrend;

    if (!iConditions)
        iConditions = new (ELeave) CArrayFixFlat<TWeatherCondition> (2);
    
    iConditions->AppendL(condition);

    }


void CWeatherInfo::AddCloudL(TCloudCoverage aCoverage, TInt aHeight)
    {

    TCloud cloud;
    
    cloud.iCoverage = aCoverage;
    cloud.iHeight = aHeight;


    if (!iClouds)
        iClouds = new (ELeave) CArrayFixFlat<TCloud> (2);
    
    iClouds->AppendL(cloud);


    }


void CWeatherInfo::SetWind(TInt aWindSpeed, TCompassDirection aWindDirection, 
                           TInt aGustSpeed, TBool aDirectionVariable)
    {

    iHasWind = ETrue;

    iWindSpeed = aWindSpeed;
    iWindDirection = aWindDirection;
    iGustSpeed = aGustSpeed;
    iDirectionVariable = aDirectionVariable;

    }



void CWeatherInfo::SetVisibility(TInt aVisibility)
    {

    iHasVisibility = ETrue;
    iVisibility = aVisibility;

    }


void CWeatherInfo::SetTemperature(TInt aTemperature)
    {

    iHasTemperature = ETrue;
    iTemperature = aTemperature;

    }


void CWeatherInfo::SetDewPoint(TInt aDewPoint)
    {

    iHasDewPoint = ETrue;
    iDewPoint= aDewPoint;

    }

void CWeatherInfo::SetPressure(TInt aPressure)
    {

    iHasPressure = ETrue;
    iPressure = aPressure;

    }

void CWeatherInfo::SetTime(TTime& aTime)
    {
    iReportTime = aTime;

    }



CArrayFixFlat<TWeatherCondition>* CWeatherInfo::GetConditions()
    {

    return iConditions;

    }

CArrayFixFlat<TCloud>* CWeatherInfo::GetClouds()
    {

    return iClouds;

    }


TBool CWeatherInfo::GetWind(TInt& aWindSpeed, TCompassDirection& aWindDirection, 
                           TInt& aGustSpeed, TBool& aDirectionVariable)
    {

    if (!iHasWind)
        return EFalse;

    aWindSpeed = iWindSpeed;
    aWindDirection = iWindDirection;
    aGustSpeed = iGustSpeed;
    aDirectionVariable = iDirectionVariable;

    return ETrue;

    }


TBool CWeatherInfo::GetVisibility(TInt& aVisibility)
    {
    if (!iHasVisibility)
        return EFalse;

    aVisibility = iVisibility;
    return ETrue;

    }

TBool CWeatherInfo::GetTemperature(TInt& aTemperature)
    {

    if (!iHasTemperature)
        return EFalse;

    
    aTemperature = iTemperature;
    return ETrue;
    }


TBool CWeatherInfo::GetDewPoint(TInt& aDewPoint)
    {

    if (!iHasDewPoint)
        return EFalse;

    aDewPoint = iDewPoint;
    return ETrue;
    }

TBool CWeatherInfo::GetPressure(TInt& aPressure)
    {
    if (!iHasPressure)
        return EFalse;

    aPressure = iPressure;
    return ETrue;


    }

TBool CWeatherInfo::GetTime(TTime& aTime)
    {
    aTime = iReportTime;
    return ETrue;

    }



