

/*
 * Copyright Justin Armstrong 2004-2009. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include "weatherutils.h"

#include <weather.rsg>
#include <e32math.h>

double MilesToMetres(TInt aMiles)
    {
    return aMiles*1609.344;
    }

double MetresToMiles(TInt aMetres)
    {
    return (double)aMetres*0.0006214;
    }


double FeetToMiles(TInt aFeet)
    {
    return (double)aFeet*0.0001894;
    }


double FeetToMetres(TInt aFeet)
    {
    return (double)aFeet*0.3048;
    }


double KphToKnots(TInt aKph)
    {
    return aKph*1.852;

    }

double MpsToKnots(TInt aMps)
    {
    return aMps*1.9438445;
    }

double KnotsToKph(TInt aKnots)
    {
    return aKnots*1.852;
    }

double KnotsToMps(TInt aKnots)
    {
    return aKnots*0.5144444;
    }

double KnotsToMph(TInt aKnots)
    {
    return aKnots*1.1507794;
    }


double MphToKnots(TInt aMph)
    {
    return aMph*0.8689762;
    }


double CentigradeToFahrenheit(TInt aCentigrade)
    {
    return (aCentigrade * 9.0/5.0+32.0);
    }



void MakeSpeedString(TDes& aString, TInt aKnots)
    {

    if (GLOBALS->iSettings.iSpeedUnits == EKph)
        {
        TBuf<40> kphBuf;
        CEikonEnv::Static()->ReadResource(kphBuf, R_TEXT_KPH);

        aString.Format(_L("%.1f %S "), KnotsToKph(aKnots), &kphBuf);
        }
    else if (GLOBALS->iSettings.iSpeedUnits == EMph)
        {
        TBuf<40> mphBuf;
        CEikonEnv::Static()->ReadResource(mphBuf, R_TEXT_MPH);

        aString.Format(_L("%.1f %S "), KnotsToMph(aKnots), &mphBuf);
        }
    else if (GLOBALS->iSettings.iSpeedUnits == EMps)
        {
        TBuf<40> mpsBuf;
        CEikonEnv::Static()->ReadResource(mpsBuf, R_TEXT_MPS);

        aString.Format(_L("%.1f %S "), KnotsToMps(aKnots), &mpsBuf);
        }
    else
        {
        TBuf<40> knotsBuf;
        CEikonEnv::Static()->ReadResource(knotsBuf, R_TEXT_KNOTS);

        aString.Format(_L("%d %S "), aKnots, &knotsBuf);
        }    

    }

void MakeTempString(TDes& aString, TInt aCentigrade)
    {

    TBuf<10> formatBuf;
    if (GLOBALS->iSettings.iTempUnits == EFahrenheit)
        {
        CEikonEnv::Static()->ReadResource(formatBuf, R_DEGREES_FAHRENHEIT_FORMAT);
        aString.Format(formatBuf, (TInt)CentigradeToFahrenheit(aCentigrade));
        }
    else
        {
        CEikonEnv::Static()->ReadResource(formatBuf, R_DEGREES_CENTIGRADE_FORMAT);
        aString.Format(formatBuf, aCentigrade);
        }

    }

// -----------------------------------------------------------


double RelativeHumidity(double aTemp, double aDewPoint)
{

    TReal e= 0.0;
    TReal es=0.0;
    double z;

    Math::Exp(e, (17.269 * aDewPoint) / (237.3 + aDewPoint));
    Math::Exp(es, (17.269 * aTemp) / (237.3 + aTemp));

    z = e / es;
    return z;


}


