

// INCLUDE FILES
#include <aknviewappui.h>
#include <aknconsts.h>
#include <aknutils.h> 


#include "weathercontainer.h" 
#include "constants.h"
#include "weather.hrh"

// ========================= MEMBER FUNCTIONS ==================================

// -----------------------------------------------------------------------------
// CWeatherViewController::CWeatherViewController()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CWeatherViewController::CWeatherViewController() : iLocation(NULL)
    {
    // No implementation required
    }


// -----------------------------------------------------------------------------
// CWeatherViewController::NewL()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CWeatherViewController* CWeatherViewController::NewL()
    {
    CWeatherViewController* self = CWeatherViewController::NewLC();
    CleanupStack::Pop( self );
    return self;
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::NewLC()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CWeatherViewController* CWeatherViewController::NewLC()
    {
    CWeatherViewController* self = new ( ELeave ) CWeatherViewController();
    CleanupStack::PushL( self );
    self->ConstructL();
    return self;
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CWeatherViewController::ConstructL()
    {
    BaseConstructL(R_WEATHER_VIEW);
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::~CWeatherViewController()
// Destructor.
// -----------------------------------------------------------------------------
//
CWeatherViewController::~CWeatherViewController()
    {
    // No implementation required
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::Id()
// Returns View's ID.
// -----------------------------------------------------------------------------
//
TUid CWeatherViewController::Id() const
    {
    return TUid::Uid( EWeatherViewId );
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::DoActivateL()
// Activate an MultiView1
// -----------------------------------------------------------------------------
//
void CWeatherViewController::DoActivateL( const TVwsViewId& /*aPrevViewId*/,
                                    TUid /*aCustomMessageId*/,
                                    const TDesC8& /*aCustomMessage*/)
    {
    iContainer = new (ELeave) CWeatherContainer();
    iContainer->SetMopParent(this);
    iContainer->ConstructL( ClientRect() );
    AppUi()->AddToStackL(iContainer);
    iContainer->SetLocation(iLocation);
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::DoDeactivate()
// DeActivate an MultiView1
// -----------------------------------------------------------------------------
//
void CWeatherViewController::DoDeactivate()
    {

    if ( iContainer )
        {
        // Remove View1's container form controllStack
        AppUi()->RemoveFromStack( iContainer );
        delete iContainer;
        iContainer = NULL;
        }
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::HandleCommandL()
// Takes care of Command handling.
// -----------------------------------------------------------------------------
//
void CWeatherViewController::HandleCommandL( TInt aCommand )
    {
/*
    if ( aCommand == EMultiViewsSwitchToView2 )
        {
        AppUi()->ActivateLocalViewL( TUid::Uid( EMultiViewsView2Id ) );
        }
    else*/
        {
        AppUi()->HandleCommandL( aCommand );
        }
    }

void CWeatherViewController::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
    {
    AppUi()->DynInitMenuPaneL(aResourceId, aMenuPane);
    }

// -----------------------------------------------------------------------------
// CWeatherViewController::HandleSizeChange()
// Called by HandleResourceChangeL() from CMultiViewsAppUi when layout is 
// changed.
// -----------------------------------------------------------------------------
//
void CWeatherViewController::HandleSizeChange( TInt aType )
    {
    if( iContainer )
        {
        iContainer->HandleResourceChange( aType );
        
        if ( aType==KEikDynamicLayoutVariantSwitch )
            {        
            TRect rect;
            AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane, rect);
            iContainer->SetRect(rect);
            }
        }         
    }
    

void CWeatherViewController::SetLocation(CLocation* aLocation)
    {
    iLocation = aLocation;
    if (iContainer)
        {
        iContainer->SetLocation(aLocation);
        }
    }

// End of File
